# pylint: skip-file
"""
Database of known variable names
NOTE: this file is automatically generated by the script
python -Bm cmakelang.tools.parse_cmake_help variables
"""
import re


PATTERNS = [
    "(?P<PROJECT_NAME>.*)_BINARY_DIR",
    "(?P<PROJECT_NAME>.*)_SOURCE_DIR",
    "(?P<PROJECT_NAME>.*)_VERSION",
    "(?P<PROJECT_NAME>.*)_VERSION_MAJOR",
    "(?P<PROJECT_NAME>.*)_VERSION_MINOR",
    "(?P<PROJECT_NAME>.*)_VERSION_PATCH",
    "(?P<PROJECT_NAME>.*)_VERSION_TWEAK",
    "ANDROID",
    "APPLE",
    "BORLAND",
    "BUILD_SHARED_LIBS",
    "CMAKE_(?P<CONFIG>.*)_POSTFIX",
    "CMAKE_(?P<LANG>.*)_ANDROID_TOOLCHAIN_MACHINE",
    "CMAKE_(?P<LANG>.*)_ANDROID_TOOLCHAIN_PREFIX",
    "CMAKE_(?P<LANG>.*)_ANDROID_TOOLCHAIN_SUFFIX",
    "CMAKE_(?P<LANG>.*)_ARCHIVE_APPEND",
    "CMAKE_(?P<LANG>.*)_ARCHIVE_CREATE",
    "CMAKE_(?P<LANG>.*)_ARCHIVE_FINISH",
    "CMAKE_(?P<LANG>.*)_CLANG_TIDY",
    "CMAKE_(?P<LANG>.*)_COMPILER",
    "CMAKE_(?P<LANG>.*)_COMPILER_ABI",
    "CMAKE_(?P<LANG>.*)_COMPILER_AR",
    "CMAKE_(?P<LANG>.*)_COMPILER_ARCHITECTURE_ID",
    "CMAKE_(?P<LANG>.*)_COMPILER_EXTERNAL_TOOLCHAIN",
    "CMAKE_(?P<LANG>.*)_COMPILER_ID",
    "CMAKE_(?P<LANG>.*)_COMPILER_LAUNCHER",
    "CMAKE_(?P<LANG>.*)_COMPILER_LOADED",
    "CMAKE_(?P<LANG>.*)_COMPILER_PREDEFINES_COMMAND",
    "CMAKE_(?P<LANG>.*)_COMPILER_RANLIB",
    "CMAKE_(?P<LANG>.*)_COMPILER_TARGET",
    "CMAKE_(?P<LANG>.*)_COMPILER_VERSION",
    "CMAKE_(?P<LANG>.*)_COMPILER_VERSION_INTERNAL",
    "CMAKE_(?P<LANG>.*)_COMPILE_OBJECT",
    "CMAKE_(?P<LANG>.*)_CPPCHECK",
    "CMAKE_(?P<LANG>.*)_CPPLINT",
    "CMAKE_(?P<LANG>.*)_CREATE_SHARED_LIBRARY",
    "CMAKE_(?P<LANG>.*)_CREATE_SHARED_MODULE",
    "CMAKE_(?P<LANG>.*)_CREATE_STATIC_LIBRARY",
    "CMAKE_(?P<LANG>.*)_FLAGS",
    "CMAKE_(?P<LANG>.*)_FLAGS_DEBUG",
    "CMAKE_(?P<LANG>.*)_FLAGS_DEBUG_INIT",
    "CMAKE_(?P<LANG>.*)_FLAGS_INIT",
    "CMAKE_(?P<LANG>.*)_FLAGS_MINSIZEREL",
    "CMAKE_(?P<LANG>.*)_FLAGS_MINSIZEREL_INIT",
    "CMAKE_(?P<LANG>.*)_FLAGS_RELEASE",
    "CMAKE_(?P<LANG>.*)_FLAGS_RELEASE_INIT",
    "CMAKE_(?P<LANG>.*)_FLAGS_RELWITHDEBINFO",
    "CMAKE_(?P<LANG>.*)_FLAGS_RELWITHDEBINFO_INIT",
    "CMAKE_(?P<LANG>.*)_GHS_KERNEL_FLAGS_DEBUG",
    "CMAKE_(?P<LANG>.*)_GHS_KERNEL_FLAGS_MINSIZEREL",
    "CMAKE_(?P<LANG>.*)_GHS_KERNEL_FLAGS_RELEASE",
    "CMAKE_(?P<LANG>.*)_GHS_KERNEL_FLAGS_RELWITHDEBINFO",
    "CMAKE_(?P<LANG>.*)_IGNORE_EXTENSIONS",
    "CMAKE_(?P<LANG>.*)_IMPLICIT_INCLUDE_DIRECTORIES",
    "CMAKE_(?P<LANG>.*)_IMPLICIT_LINK_DIRECTORIES",
    "CMAKE_(?P<LANG>.*)_IMPLICIT_LINK_FRAMEWORK_DIRECTORIES",
    "CMAKE_(?P<LANG>.*)_IMPLICIT_LINK_LIBRARIES",
    "CMAKE_(?P<LANG>.*)_INCLUDE_WHAT_YOU_USE",
    "CMAKE_(?P<LANG>.*)_LIBRARY_ARCHITECTURE",
    "CMAKE_(?P<LANG>.*)_LINKER_PREFERENCE",
    "CMAKE_(?P<LANG>.*)_LINKER_PREFERENCE_PROPAGATES",
    "CMAKE_(?P<LANG>.*)_LINK_EXECUTABLE",
    "CMAKE_(?P<LANG>.*)_OUTPUT_EXTENSION",
    "CMAKE_(?P<LANG>.*)_PLATFORM_ID",
    "CMAKE_(?P<LANG>.*)_SIMULATE_ID",
    "CMAKE_(?P<LANG>.*)_SIMULATE_VERSION",
    "CMAKE_(?P<LANG>.*)_SIZEOF_DATA_PTR",
    "CMAKE_(?P<LANG>.*)_SOURCE_FILE_EXTENSIONS",
    "CMAKE_(?P<LANG>.*)_STANDARD_INCLUDE_DIRECTORIES",
    "CMAKE_(?P<LANG>.*)_STANDARD_LIBRARIES",
    "CMAKE_(?P<LANG>.*)_VISIBILITY_PRESET",
    "CMAKE_ABSOLUTE_DESTINATION_FILES",
    "CMAKE_ANDROID_ANT_ADDITIONAL_OPTIONS",
    "CMAKE_ANDROID_API",
    "CMAKE_ANDROID_API_MIN",
    "CMAKE_ANDROID_ARCH",
    "CMAKE_ANDROID_ARCH_ABI",
    "CMAKE_ANDROID_ARM_MODE",
    "CMAKE_ANDROID_ARM_NEON",
    "CMAKE_ANDROID_ASSETS_DIRECTORIES",
    "CMAKE_ANDROID_GUI",
    "CMAKE_ANDROID_JAR_DEPENDENCIES",
    "CMAKE_ANDROID_JAR_DIRECTORIES",
    "CMAKE_ANDROID_JAVA_SOURCE_DIR",
    "CMAKE_ANDROID_NATIVE_LIB_DEPENDENCIES",
    "CMAKE_ANDROID_NATIVE_LIB_DIRECTORIES",
    "CMAKE_ANDROID_NDK",
    "CMAKE_ANDROID_NDK_DEPRECATED_HEADERS",
    "CMAKE_ANDROID_NDK_TOOLCHAIN_HOST_TAG",
    "CMAKE_ANDROID_NDK_TOOLCHAIN_VERSION",
    "CMAKE_ANDROID_PROCESS_MAX",
    "CMAKE_ANDROID_PROGUARD",
    "CMAKE_ANDROID_PROGUARD_CONFIG_PATH",
    "CMAKE_ANDROID_SECURE_PROPS_PATH",
    "CMAKE_ANDROID_SKIP_ANT_STEP",
    "CMAKE_ANDROID_STANDALONE_TOOLCHAIN",
    "CMAKE_ANDROID_STL_TYPE",
    "CMAKE_APPBUNDLE_PATH",
    "CMAKE_AR",
    "CMAKE_ARCHIVE_OUTPUT_DIRECTORY",
    "CMAKE_ARCHIVE_OUTPUT_DIRECTORY_(?P<CONFIG>.*)",
    "CMAKE_ARGC",
    "CMAKE_ARGV0",
    "CMAKE_AUTOMOC",
    "CMAKE_AUTOMOC_COMPILER_PREDEFINES",
    "CMAKE_AUTOMOC_DEPEND_FILTERS",
    "CMAKE_AUTOMOC_MACRO_NAMES",
    "CMAKE_AUTOMOC_MOC_OPTIONS",
    "CMAKE_AUTOMOC_RELAXED_MODE",
    "CMAKE_AUTORCC",
    "CMAKE_AUTORCC_OPTIONS",
    "CMAKE_AUTOUIC",
    "CMAKE_AUTOUIC_OPTIONS",
    "CMAKE_AUTOUIC_SEARCH_PATHS",
    "CMAKE_BACKWARDS_COMPATIBILITY",
    "CMAKE_BINARY_DIR",
    "CMAKE_BUILD_RPATH",
    "CMAKE_BUILD_TOOL",
    "CMAKE_BUILD_TYPE",
    "CMAKE_BUILD_WITH_INSTALL_NAME_DIR",
    "CMAKE_BUILD_WITH_INSTALL_RPATH",
    "CMAKE_CACHEFILE_DIR",
    "CMAKE_CACHE_MAJOR_VERSION",
    "CMAKE_CACHE_MINOR_VERSION",
    "CMAKE_CACHE_PATCH_VERSION",
    "CMAKE_CFG_INTDIR",
    "CMAKE_CL_64",
    "CMAKE_CODEBLOCKS_EXCLUDE_EXTERNAL_FILES",
    "CMAKE_CODELITE_USE_TARGETS",
    "CMAKE_COLOR_MAKEFILE",
    "CMAKE_COMMAND",
    "CMAKE_COMPILER_2005",
    "CMAKE_COMPILER_IS_GNUCC",
    "CMAKE_COMPILER_IS_GNUCXX",
    "CMAKE_COMPILER_IS_GNUG77",
    "CMAKE_COMPILE_PDB_OUTPUT_DIRECTORY",
    "CMAKE_COMPILE_PDB_OUTPUT_DIRECTORY_(?P<CONFIG>.*)",
    "CMAKE_CONFIGURATION_TYPES",
    "CMAKE_CROSSCOMPILING",
    "CMAKE_CROSSCOMPILING_EMULATOR",
    "CMAKE_CTEST_COMMAND",
    "CMAKE_CUDA_EXTENSIONS",
    "CMAKE_CUDA_HOST_COMPILER",
    "CMAKE_CUDA_STANDARD",
    "CMAKE_CUDA_STANDARD_REQUIRED",
    "CMAKE_CUDA_TOOLKIT_INCLUDE_DIRECTORIES",
    "CMAKE_CURRENT_BINARY_DIR",
    "CMAKE_CURRENT_LIST_DIR",
    "CMAKE_CURRENT_LIST_FILE",
    "CMAKE_CURRENT_LIST_LINE",
    "CMAKE_CURRENT_SOURCE_DIR",
    "CMAKE_CXX_COMPILE_FEATURES",
    "CMAKE_CXX_EXTENSIONS",
    "CMAKE_CXX_STANDARD",
    "CMAKE_CXX_STANDARD_REQUIRED",
    "CMAKE_C_COMPILE_FEATURES",
    "CMAKE_C_EXTENSIONS",
    "CMAKE_C_STANDARD",
    "CMAKE_C_STANDARD_REQUIRED",
    "CMAKE_DEBUG_POSTFIX",
    "CMAKE_DEBUG_TARGET_PROPERTIES",
    "CMAKE_DEPENDS_IN_PROJECT_ONLY",
    "CMAKE_DIRECTORY_LABELS",
    "CMAKE_DISABLE_FIND_PACKAGE_(?P<PackageName>.*)",
    "CMAKE_DL_LIBS",
    "CMAKE_ECLIPSE_GENERATE_LINKED_RESOURCES",
    "CMAKE_ECLIPSE_GENERATE_SOURCE_PROJECT",
    "CMAKE_ECLIPSE_MAKE_ARGUMENTS",
    "CMAKE_ECLIPSE_VERSION",
    "CMAKE_EDIT_COMMAND",
    "CMAKE_ENABLE_EXPORTS",
    "CMAKE_ERROR_DEPRECATED",
    "CMAKE_ERROR_ON_ABSOLUTE_INSTALL_DESTINATION",
    "CMAKE_EXECUTABLE_SUFFIX",
    "CMAKE_EXE_LINKER_FLAGS",
    "CMAKE_EXE_LINKER_FLAGS_(?P<CONFIG>.*)",
    "CMAKE_EXE_LINKER_FLAGS_(?P<CONFIG>.*)_INIT",
    "CMAKE_EXE_LINKER_FLAGS_INIT",
    "CMAKE_EXPORT_COMPILE_COMMANDS",
    "CMAKE_EXPORT_NO_PACKAGE_REGISTRY",
    "CMAKE_EXTRA_GENERATOR",
    "CMAKE_EXTRA_SHARED_LIBRARY_SUFFIXES",
    "CMAKE_FIND_APPBUNDLE",
    "CMAKE_FIND_FRAMEWORK",
    "CMAKE_FIND_LIBRARY_CUSTOM_LIB_SUFFIX",
    "CMAKE_FIND_LIBRARY_PREFIXES",
    "CMAKE_FIND_LIBRARY_SUFFIXES",
    "CMAKE_FIND_NO_INSTALL_PREFIX",
    "CMAKE_FIND_PACKAGE_NAME",
    "CMAKE_FIND_PACKAGE_NO_PACKAGE_REGISTRY",
    "CMAKE_FIND_PACKAGE_NO_SYSTEM_PACKAGE_REGISTRY",
    "CMAKE_FIND_PACKAGE_SORT_DIRECTION",
    "CMAKE_FIND_PACKAGE_SORT_ORDER",
    "CMAKE_FIND_PACKAGE_WARN_NO_MODULE",
    "CMAKE_FIND_ROOT_PATH",
    "CMAKE_FIND_ROOT_PATH_MODE_INCLUDE",
    "CMAKE_FIND_ROOT_PATH_MODE_LIBRARY",
    "CMAKE_FIND_ROOT_PATH_MODE_PACKAGE",
    "CMAKE_FIND_ROOT_PATH_MODE_PROGRAM",
    "CMAKE_FRAMEWORK_PATH",
    "CMAKE_Fortran_FORMAT",
    "CMAKE_Fortran_MODDIR_DEFAULT",
    "CMAKE_Fortran_MODDIR_FLAG",
    "CMAKE_Fortran_MODOUT_FLAG",
    "CMAKE_Fortran_MODULE_DIRECTORY",
    "CMAKE_GENERATOR",
    "CMAKE_GENERATOR_PLATFORM",
    "CMAKE_GENERATOR_TOOLSET",
    "CMAKE_GNUtoMS",
    "CMAKE_HOME_DIRECTORY",
    "CMAKE_HOST_APPLE",
    "CMAKE_HOST_SOLARIS",
    "CMAKE_HOST_SYSTEM",
    "CMAKE_HOST_SYSTEM_NAME",
    "CMAKE_HOST_SYSTEM_PROCESSOR",
    "CMAKE_HOST_SYSTEM_VERSION",
    "CMAKE_HOST_UNIX",
    "CMAKE_HOST_WIN32",
    "CMAKE_IGNORE_PATH",
    "CMAKE_IMPORT_LIBRARY_PREFIX",
    "CMAKE_IMPORT_LIBRARY_SUFFIX",
    "CMAKE_INCLUDE_CURRENT_DIR",
    "CMAKE_INCLUDE_CURRENT_DIR_IN_INTERFACE",
    "CMAKE_INCLUDE_DIRECTORIES_BEFORE",
    "CMAKE_INCLUDE_DIRECTORIES_PROJECT_BEFORE",
    "CMAKE_INCLUDE_PATH",
    "CMAKE_INSTALL_DEFAULT_COMPONENT_NAME",
    "CMAKE_INSTALL_MESSAGE",
    "CMAKE_INSTALL_NAME_DIR",
    "CMAKE_INSTALL_PREFIX",
    "CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT",
    "CMAKE_INSTALL_RPATH",
    "CMAKE_INSTALL_RPATH_USE_LINK_PATH",
    "CMAKE_INTERNAL_PLATFORM_ABI",
    "CMAKE_INTERPROCEDURAL_OPTIMIZATION",
    "CMAKE_INTERPROCEDURAL_OPTIMIZATION_(?P<CONFIG>.*)",
    "CMAKE_IOS_INSTALL_COMBINED",
    "CMAKE_JOB_POOL_COMPILE",
    "CMAKE_JOB_POOL_LINK",
    "CMAKE_LIBRARY_ARCHITECTURE",
    "CMAKE_LIBRARY_ARCHITECTURE_REGEX",
    "CMAKE_LIBRARY_OUTPUT_DIRECTORY",
    "CMAKE_LIBRARY_OUTPUT_DIRECTORY_(?P<CONFIG>.*)",
    "CMAKE_LIBRARY_PATH",
    "CMAKE_LIBRARY_PATH_FLAG",
    "CMAKE_LINK_DEF_FILE_FLAG",
    "CMAKE_LINK_DEPENDS_NO_SHARED",
    "CMAKE_LINK_INTERFACE_LIBRARIES",
    "CMAKE_LINK_LIBRARY_FILE_FLAG",
    "CMAKE_LINK_LIBRARY_FLAG",
    "CMAKE_LINK_LIBRARY_SUFFIX",
    "CMAKE_LINK_SEARCH_END_STATIC",
    "CMAKE_LINK_SEARCH_START_STATIC",
    "CMAKE_LINK_WHAT_YOU_USE",
    "CMAKE_MACOSX_BUNDLE",
    "CMAKE_MACOSX_RPATH",
    "CMAKE_MAJOR_VERSION",
    "CMAKE_MAKE_PROGRAM",
    "CMAKE_MAP_IMPORTED_CONFIG_(?P<CONFIG>.*)",
    "CMAKE_MATCH_(?P<n>.*)",
    "CMAKE_MATCH_COUNT",
    "CMAKE_MFC_FLAG",
    "CMAKE_MINIMUM_REQUIRED_VERSION",
    "CMAKE_MINOR_VERSION",
    "CMAKE_MODULE_LINKER_FLAGS",
    "CMAKE_MODULE_LINKER_FLAGS_(?P<CONFIG>.*)",
    "CMAKE_MODULE_LINKER_FLAGS_(?P<CONFIG>.*)_INIT",
    "CMAKE_MODULE_LINKER_FLAGS_INIT",
    "CMAKE_MODULE_PATH",
    "CMAKE_MSVCIDE_RUN_PATH",
    "CMAKE_NINJA_OUTPUT_PATH_PREFIX",
    "CMAKE_NOT_USING_CONFIG_FLAGS",
    "CMAKE_NO_BUILTIN_CHRPATH",
    "CMAKE_NO_SYSTEM_FROM_IMPORTED",
    "CMAKE_OBJECT_PATH_MAX",
    "CMAKE_OSX_ARCHITECTURES",
    "CMAKE_OSX_DEPLOYMENT_TARGET",
    "CMAKE_OSX_SYSROOT",
    "CMAKE_PARENT_LIST_FILE",
    "CMAKE_PATCH_VERSION",
    "CMAKE_PDB_OUTPUT_DIRECTORY",
    "CMAKE_PDB_OUTPUT_DIRECTORY_(?P<CONFIG>.*)",
    "CMAKE_POLICY_DEFAULT_CMP(?P<NNNN>.*)",
    "CMAKE_POLICY_WARNING_CMP(?P<NNNN>.*)",
    "CMAKE_POSITION_INDEPENDENT_CODE",
    "CMAKE_PREFIX_PATH",
    "CMAKE_PROGRAM_PATH",
    "CMAKE_PROJECT_(?P<PROJECT_NAME>.*)_INCLUDE",
    "CMAKE_PROJECT_DESCRIPTION",
    "CMAKE_PROJECT_NAME",
    "CMAKE_RANLIB",
    "CMAKE_ROOT",
    "CMAKE_RUNTIME_OUTPUT_DIRECTORY",
    "CMAKE_RUNTIME_OUTPUT_DIRECTORY_(?P<CONFIG>.*)",
    "CMAKE_SCRIPT_MODE_FILE",
    "CMAKE_SHARED_LIBRARY_PREFIX",
    "CMAKE_SHARED_LIBRARY_SUFFIX",
    "CMAKE_SHARED_LINKER_FLAGS",
    "CMAKE_SHARED_LINKER_FLAGS_(?P<CONFIG>.*)",
    "CMAKE_SHARED_LINKER_FLAGS_(?P<CONFIG>.*)_INIT",
    "CMAKE_SHARED_LINKER_FLAGS_INIT",
    "CMAKE_SHARED_MODULE_PREFIX",
    "CMAKE_SHARED_MODULE_SUFFIX",
    "CMAKE_SIZEOF_VOID_P",
    "CMAKE_SKIP_BUILD_RPATH",
    "CMAKE_SKIP_INSTALL_ALL_DEPENDENCY",
    "CMAKE_SKIP_INSTALL_RPATH",
    "CMAKE_SKIP_INSTALL_RULES",
    "CMAKE_SKIP_RPATH",
    "CMAKE_SOURCE_DIR",
    "CMAKE_STAGING_PREFIX",
    "CMAKE_STATIC_LIBRARY_PREFIX",
    "CMAKE_STATIC_LIBRARY_SUFFIX",
    "CMAKE_STATIC_LINKER_FLAGS",
    "CMAKE_STATIC_LINKER_FLAGS_(?P<CONFIG>.*)",
    "CMAKE_STATIC_LINKER_FLAGS_(?P<CONFIG>.*)_INIT",
    "CMAKE_STATIC_LINKER_FLAGS_INIT",
    "CMAKE_SUBLIME_TEXT_2_ENV_SETTINGS",
    "CMAKE_SUBLIME_TEXT_2_EXCLUDE_BUILD_TREE",
    "CMAKE_SYSROOT",
    "CMAKE_SYSROOT_COMPILE",
    "CMAKE_SYSROOT_LINK",
    "CMAKE_SYSTEM",
    "CMAKE_SYSTEM_APPBUNDLE_PATH",
    "CMAKE_SYSTEM_FRAMEWORK_PATH",
    "CMAKE_SYSTEM_IGNORE_PATH",
    "CMAKE_SYSTEM_INCLUDE_PATH",
    "CMAKE_SYSTEM_LIBRARY_PATH",
    "CMAKE_SYSTEM_NAME",
    "CMAKE_SYSTEM_PREFIX_PATH",
    "CMAKE_SYSTEM_PROCESSOR",
    "CMAKE_SYSTEM_PROGRAM_PATH",
    "CMAKE_SYSTEM_VERSION",
    "CMAKE_Swift_LANGUAGE_VERSION",
    "CMAKE_TOOLCHAIN_FILE",
    "CMAKE_TRY_COMPILE_CONFIGURATION",
    "CMAKE_TRY_COMPILE_PLATFORM_VARIABLES",
    "CMAKE_TRY_COMPILE_TARGET_TYPE",
    "CMAKE_TWEAK_VERSION",
    "CMAKE_USER_MAKE_RULES_OVERRIDE",
    "CMAKE_USER_MAKE_RULES_OVERRIDE_(?P<LANG>.*)",
    "CMAKE_USE_RELATIVE_PATHS",
    "CMAKE_VERBOSE_MAKEFILE",
    "CMAKE_VERSION",
    "CMAKE_VISIBILITY_INLINES_HIDDEN",
    "CMAKE_VS_DEVENV_COMMAND",
    "CMAKE_VS_INCLUDE_INSTALL_TO_DEFAULT_BUILD",
    "CMAKE_VS_INCLUDE_PACKAGE_TO_DEFAULT_BUILD",
    "CMAKE_VS_INTEL_Fortran_PROJECT_VERSION",
    "CMAKE_VS_MSBUILD_COMMAND",
    "CMAKE_VS_NsightTegra_VERSION",
    "CMAKE_VS_PLATFORM_NAME",
    "CMAKE_VS_PLATFORM_TOOLSET",
    "CMAKE_VS_PLATFORM_TOOLSET_CUDA",
    "CMAKE_VS_PLATFORM_TOOLSET_HOST_ARCHITECTURE",
    "CMAKE_VS_WINDOWS_TARGET_PLATFORM_VERSION",
    "CMAKE_WARN_DEPRECATED",
    "CMAKE_WARN_ON_ABSOLUTE_INSTALL_DESTINATION",
    "CMAKE_WIN32_EXECUTABLE",
    "CMAKE_WINDOWS_EXPORT_ALL_SYMBOLS",
    "CMAKE_XCODE_ATTRIBUTE_(?P<an_attribute>.*)",
    "CMAKE_XCODE_GENERATE_SCHEME",
    "CMAKE_XCODE_PLATFORM_TOOLSET",
    "CPACK_ABSOLUTE_DESTINATION_FILES",
    "CPACK_COMPONENT_INCLUDE_TOPLEVEL_DIRECTORY",
    "CPACK_ERROR_ON_ABSOLUTE_INSTALL_DESTINATION",
    "CPACK_INCLUDE_TOPLEVEL_DIRECTORY",
    "CPACK_INSTALL_SCRIPT",
    "CPACK_PACKAGING_INSTALL_PREFIX",
    "CPACK_SET_DESTDIR",
    "CPACK_WARN_ON_ABSOLUTE_INSTALL_DESTINATION",
    "CTEST_BINARY_DIRECTORY",
    "CTEST_BUILD_COMMAND",
    "CTEST_BUILD_NAME",
    "CTEST_BZR_COMMAND",
    "CTEST_BZR_UPDATE_OPTIONS",
    "CTEST_CHANGE_ID",
    "CTEST_CHECKOUT_COMMAND",
    "CTEST_CONFIGURATION_TYPE",
    "CTEST_CONFIGURE_COMMAND",
    "CTEST_COVERAGE_COMMAND",
    "CTEST_COVERAGE_EXTRA_FLAGS",
    "CTEST_CURL_OPTIONS",
    "CTEST_CUSTOM_COVERAGE_EXCLUDE",
    "CTEST_CUSTOM_ERROR_EXCEPTION",
    "CTEST_CUSTOM_ERROR_MATCH",
    "CTEST_CUSTOM_ERROR_POST_CONTEXT",
    "CTEST_CUSTOM_ERROR_PRE_CONTEXT",
    "CTEST_CUSTOM_MAXIMUM_FAILED_TEST_OUTPUT_SIZE",
    "CTEST_CUSTOM_MAXIMUM_NUMBER_OF_ERRORS",
    "CTEST_CUSTOM_MAXIMUM_NUMBER_OF_WARNINGS",
    "CTEST_CUSTOM_MAXIMUM_PASSED_TEST_OUTPUT_SIZE",
    "CTEST_CUSTOM_MEMCHECK_IGNORE",
    "CTEST_CUSTOM_POST_MEMCHECK",
    "CTEST_CUSTOM_POST_TEST",
    "CTEST_CUSTOM_PRE_MEMCHECK",
    "CTEST_CUSTOM_PRE_TEST",
    "CTEST_CUSTOM_TEST_IGNORE",
    "CTEST_CUSTOM_WARNING_EXCEPTION",
    "CTEST_CUSTOM_WARNING_MATCH",
    "CTEST_CVS_CHECKOUT",
    "CTEST_CVS_COMMAND",
    "CTEST_CVS_UPDATE_OPTIONS",
    "CTEST_DROP_LOCATION",
    "CTEST_DROP_METHOD",
    "CTEST_DROP_SITE",
    "CTEST_DROP_SITE_CDASH",
    "CTEST_DROP_SITE_PASSWORD",
    "CTEST_DROP_SITE_USER",
    "CTEST_EXTRA_COVERAGE_GLOB",
    "CTEST_GIT_COMMAND",
    "CTEST_GIT_INIT_SUBMODULES",
    "CTEST_GIT_UPDATE_CUSTOM",
    "CTEST_GIT_UPDATE_OPTIONS",
    "CTEST_HG_COMMAND",
    "CTEST_HG_UPDATE_OPTIONS",
    "CTEST_LABELS_FOR_SUBPROJECTS",
    "CTEST_MEMORYCHECK_COMMAND",
    "CTEST_MEMORYCHECK_COMMAND_OPTIONS",
    "CTEST_MEMORYCHECK_SANITIZER_OPTIONS",
    "CTEST_MEMORYCHECK_SUPPRESSIONS_FILE",
    "CTEST_MEMORYCHECK_TYPE",
    "CTEST_NIGHTLY_START_TIME",
    "CTEST_P4_CLIENT",
    "CTEST_P4_COMMAND",
    "CTEST_P4_OPTIONS",
    "CTEST_P4_UPDATE_OPTIONS",
    "CTEST_SCP_COMMAND",
    "CTEST_SITE",
    "CTEST_SOURCE_DIRECTORY",
    "CTEST_SVN_COMMAND",
    "CTEST_SVN_OPTIONS",
    "CTEST_SVN_UPDATE_OPTIONS",
    "CTEST_TEST_LOAD",
    "CTEST_TEST_TIMEOUT",
    "CTEST_TRIGGER_SITE",
    "CTEST_UPDATE_COMMAND",
    "CTEST_UPDATE_OPTIONS",
    "CTEST_UPDATE_VERSION_ONLY",
    "CTEST_USE_LAUNCHERS",
    "CYGWIN",
    "ENV",
    "EXECUTABLE_OUTPUT_PATH",
    "GHS-MULTI",
    "LIBRARY_OUTPUT_PATH",
    "MINGW",
    "MSVC",
    "MSVC10",
    "MSVC11",
    "MSVC12",
    "MSVC14",
    "MSVC60",
    "MSVC70",
    "MSVC71",
    "MSVC80",
    "MSVC90",
    "MSVC_IDE",
    "MSVC_VERSION",
    "PROJECT_BINARY_DIR",
    "PROJECT_DESCRIPTION",
    "PROJECT_NAME",
    "PROJECT_SOURCE_DIR",
    "PROJECT_VERSION",
    "PROJECT_VERSION_MAJOR",
    "PROJECT_VERSION_MINOR",
    "PROJECT_VERSION_PATCH",
    "PROJECT_VERSION_TWEAK",
    "UNIX",
    "WIN32",
    "WINCE",
    "WINDOWS_PHONE",
    "WINDOWS_STORE",
    "XCODE",
    "XCODE_VERSION",
]


def stripped_patterns():
  """
  Remove named groups from patterns so we can join them together without
  a name collision.
  """
  regex = re.compile(r"\?P<[\w_]+>")
  return [regex.sub("", pattern) for pattern in PATTERNS]


CASE_SENSITIVE_REGEX = re.compile(
    "|".join(stripped_patterns()))
CASE_INSENSITIVE_REGEX = re.compile(
    "|".join(stripped_patterns()), re.IGNORECASE)



#!/usr/bin/env python

import argparse
import collections
import io
import logging
import os
import string
import sys

if __name__ == "__main__":
  # pylint: disable=import-error
  sys.path.append('/usr/share/gtk-doc/python')
  from gtkdoc import scangobj

logger = logging.getLogger(__name__)

TypesInfo = collections.namedtuple(
    "TypesInfo", ["includes", "forward_decls", "get_types"])


def parse_types(typespath):
  includes = []
  forward_decls = []
  get_types = []

  with io.open(typespath, "r", encoding="utf-8") as infile:
    for line in infile:
      line = line.strip()

      # comment or empty line
      if line.startswith("%") or not line.strip():
        continue

      # include line
      if line.startswith("#include"):
        includes.append(line)
        continue

      forward_decls.append('extern GType {}(void);'.format(line))
      get_types.append('  object_types[i++] = {}();'.format(line))

  return TypesInfo(includes, forward_decls, get_types)


def scangobj_run(args, outfile):
  typesinfo = parse_types(args.types)

  outfile.write(scangobj.COMMON_INCLUDES)
  outfile.write("\n".join(typesinfo.includes))
  outfile.write("\n".join(typesinfo.forward_decls))

  if args.query_child_properties:
    outfile.write(
        scangobj.QUERY_CHILD_PROPS_PROTOTYPE & args.query_child_properties)

  # substitute local vars in the template
  type_init_func = args.type_init_func
  main_func_params = "int argc, char *argv[]"
  if "argc" in type_init_func and "argv" not in type_init_func:
    main_func_params = "int argc, G_GNUC_UNUSED char *argv[]"
  elif "argc" not in type_init_func and "argv" in type_init_func:
    main_func_params = "G_GNUC_UNUSED int argc, char *argv[]"
  elif "argc" not in type_init_func and "argv" not in type_init_func:
    main_func_params = "void"

  tplargs = {
      "get_types": "\n".join(typesinfo.get_types),
      "ntypes": len(typesinfo.get_types),
      "main_func_params": main_func_params,
      "type_init_func": type_init_func,
  }

  for name in ("signals", "hierarchy", "interfaces", "prerequisites", "args"):
    tplargs["new_{}_filename".format(name)] = "{}.{}".format(args.module, name)

  main_content = string.Template(scangobj.MAIN_CODE).substitute(tplargs)
  outfile.write(main_content)

  if args.query_child_properties:
    outfile.write(
        scangobj.QUERY_CHILD_PROPS_CODE & args.query_child_properties)

  outfile.write(scangobj.MAIN_CODE_END)


def scangobj_main(args):
  """
  Replicate scangobj.run() with some simplifications and only up through the
  generation of the source code for <module>-scan.c. Don't compile and run
  the program.
  """
  outfile_path = os.path.join(
      args.output_dir, "{}-scan.c".format(args.module))

  with open(outfile_path, "w") as outfile:
    scangobj_run(args, outfile)


def setup_parser(argparser):
  subparsers = argparser.add_subparsers(dest="command")

  parser = subparsers.add_parser("scangobj")
  parser.add_argument(
      '--module', required=True,
      help='Name of the doc module being parsed')
  parser.add_argument(
      '--types', default='',
      help='The name of the file to store the types in')
  parser.add_argument(
      '--type-init-func', default='',
      help='The init function(s) to call instead of g_type_init()')
  parser.add_argument(
      '--query-child-properties', default='',
      help='A function that returns a list of child properties for a class')
  parser.add_argument(
      '--output-dir', default='.',
      help='The directory where the results are stored')


def main():
  parser = argparse.ArgumentParser(description=__doc__)
  setup_parser(parser)
  args = parser.parse_args()

  if args.command == "scangobj":
    return scangobj_main(args)

  logger.warning("Unrecognized command %s", args.command)
  return 1


if __name__ == "__main__":
  sys.exit(main())



"""
Ensure that changes to cmake-format don't screw current users
"""

# -*- coding: utf-8 -*-
# pylint: disable=R1708
from __future__ import print_function
from __future__ import unicode_literals

import os
import re
import shutil
import sys
import subprocess
import tempfile
import unittest


def iter_filelist(rootdir, include_patterns, exclude_patterns):
  """
  Walk a directory tree and yield each file matching the include patterns
  but not matching the exclude patterns.
  """
  include_patterns = [re.compile(pattern) for pattern in include_patterns]
  exclude_patterns = [re.compile(pattern) for pattern in exclude_patterns]

  for cwd, dirnames, filenames in os.walk(rootdir):
    relpath_cwd = os.path.relpath(cwd, rootdir)
    if relpath_cwd == ".":
      relpath_cwd = ""

    exclude_cwd = any(pattern.match(relpath_cwd)
                      for pattern in exclude_patterns)
    if exclude_cwd:
      dirnames[:] = []
      continue

    dirnames[:] = sorted(dirnames)
    for filename in filenames:
      relpath_filename = os.path.join(relpath_cwd, filename)
      include_filename = (
          any(pattern.match(filename) for pattern in include_patterns) or
          any(pattern.match(relpath_filename) for pattern in include_patterns)
      )
      exclude_filename = (
          any(pattern.match(filename) for pattern in exclude_patterns) or
          any(pattern.match(relpath_filename) for pattern in exclude_patterns)
      )

      if include_filename and not exclude_filename:
        yield relpath_filename


class WrapTestWithRunFun(object):
  """
  Given a instance of a bound test-method from a TestCase, wrap that with
  a callable that first calls the method, and then calls `
  test.assertCanFormat()`. Which is really just an opaque way of
  automatically calling `rest.assertCanFormat()` at the end of each test
  method.
  """

  def __init__(self, test_object, bound_method):
    self.test_object = test_object
    self.bound_method = bound_method

  def __call__(self):
    self.bound_method()
    self.test_object.assertCanFormat()


class DontScrewUsers(unittest.TestCase):
  """
  Given a bunch of public repositories, check that cmake-format runs on
  all of them.
  """

  TMPDIR = None

  def __init__(self, *args, **kwargs):
    super(DontScrewUsers, self).__init__(*args, **kwargs)
    self.repository = None
    self.clonedir = None
    self.configpath = None
    self.include_patterns = [r".*\.cmake$", "CMakeLists.txt"]
    self.exclude_patterns = [r"\.git", r"\.build", r".*third_party.*"]
    self.last_known_good = None

    # NOTE(josh): hacky introspective way of automatically calling
    # assertExpectations() at the end of every test_XXX() function
    for name in dir(self):
      if not name.startswith("test_"):
        continue
      value = getattr(self, name)
      if callable(value):
        setattr(self, name, WrapTestWithRunFun(self, value))

  @classmethod
  def setUpClass(cls):
    if os.environ.get("CMFSCREWTEST_REUSE"):
      cls.TMPDIR = os.path.join(tempfile.gettempdir(), "cmf-unscrew")
      if not os.path.exists(cls.TMPDIR):
        os.makedirs(cls.TMPDIR)
    else:
      cls.TMPDIR = tempfile.mkdtemp(prefix="cmf-unscrew_")

  @classmethod
  def tearDownClass(cls):
    if not os.environ.get("CMFSCREWTEST_REUSE"):
      shutil.rmtree(cls.TMPDIR)

  def assertCanFormat(self):
    if self.clonedir is None:
      self.clonedir = self.repository.rsplit("/", 1)[1]
    clonepath = os.path.join(self.TMPDIR, self.clonedir)
    if not os.path.exists(os.path.join(clonepath, ".git")):
      subprocess.check_call(["git", "clone", "--depth", "1",
                             self.repository, self.clonedir], cwd=self.TMPDIR)

    env = os.environ.copy()
    thisdir = os.path.dirname(__file__)
    thisdir = os.path.dirname(thisdir)
    repodir = os.path.dirname(thisdir)
    env["PYTHONPATH"] = os.path.realpath(repodir)

    fmtcmd = [sys.executable, "-Bm", "cmakelang"]
    if self.configpath:
      fmtcmd += ["--config", self.configpath]
    fmtcmd += ["-i"]

    pathiter = iter_filelist(
        clonepath,
        self.include_patterns,
        self.exclude_patterns)

    nsuccess = 0
    failed_files = []
    for relpath in pathiter:
      result = subprocess.call(fmtcmd + [relpath], cwd=clonepath, env=env)
      if result == 0:
        nsuccess += 1
      else:
        failed_files.append(relpath)

    print("{}: Successfully ran on {} files".format(self.clonedir, nsuccess))
    if failed_files:
      print("  " + "\n  ".join(failed_files))

    self.assertFalse(failed_files)

  def test_thisrepo(self):
    thisdir = os.path.realpath(os.path.dirname(__file__))
    self.repository = os.sep.join(thisdir.split(os.sep)[:-2])
    self.configpath = ".cmake-format.py"
    self.last_known_good = "efb0d6256fcbc4a3cfad9f5ebdaa34172d230ea2"
    self.exclude_patterns += [
        r"test_latin.*\.cmake"
    ]

  def test_elektra(self):
    self.repository = "https://github.com/sanssecours/elektra"
    self.configpath = ".cmake-format.yaml"
    self.last_known_good = "483752d182bc12546a2902989d63ea1cfcaae861"

  def test_pcl(self):
    self.repository = "https://github.com/PointCloudLibrary/pcl"
    self.last_known_good = "711b12144e2cb250fcbccf56b0c523a1d279297a"

  def test_arrow(self):
    self.repository = "https://github.com/apache/arrow"
    self.configpath = "cmake-format.py"
    self.last_known_good = "e32c0b3279df76d68c93724f476dfcf0c1c44fa5"
    self.exclude_patterns += [
        r".*\.h\.cmake",
        r".*Dockerfile\.cmake",
    ]

  def test_aom(self):
    self.repository = "https://aomedia.googlesource.com/aom"
    self.configpath = ".cmake-format.py"
    self.last_known_good = "81259cf147effcd93fc50fca768ade453fce0bfe"

  def test_vulkan_tools(self):
    self.repository = "https://github.com/KhronosGroup/Vulkan-Tools"
    self.configpath = ".cmake-format.py"
    self.last_known_good = "4c6673b2973a7434716a9325a0b22861beed9244"

  def test_vulkan_validation_layers(self):
    self.repository = "https://github.com/KhronosGroup/Vulkan-ValidationLayers"
    self.configpath = "scripts/cmake-format.py"
    self.last_known_good = "adbfa85caafb5f240474dd2ae2414f0b78026a3f"
    self.exclude_patterns += [
        ".*build-android/cmake/layerlib/CMakeLists.txt"
    ]

  def test_vulkan_validation_loader(self):
    self.repository = "https://github.com/KhronosGroup/Vulkan-Loader"
    self.configpath = ".cmake-format.py"

  def test_pdfslicer(self):
    self.repository = "https://github.com/junrrein/pdfslicer"
    self.configpath = ".cmake-format.py"

  def test_opencensus(self):
    self.repository = "https://github.com/census-instrumentation/opencensus-cpp"
    self.configpath = ".cmake-format.py"

  def test_openbmc_ipmbbridge(self):
    self.repository = "https://github.com/openbmc/ipmbbridge"
    self.configpath = "cmake-format.json"

  def test_openbmc_dbus_sensors(self):
    self.repository = "https://github.com/openbmc/dbus-sensors"
    self.configpath = "cmake-format.json"


if __name__ == "__main__":
  unittest.main()



"""
Get the wheel filename without building the wheel.
"""
from __future__ import print_function

import argparse
import io

from setuptools import Extension
from setuptools.dist import Distribution


def main():
  parser = argparse.ArgumentParser(description=__doc__)
  parser.add_argument("--name", required=True)
  parser.add_argument("--version-from", required=True)
  parser.add_argument("--has-extension-module", action="store_true")
  args = parser.parse_args()

  version = None
  with io.open(args.version_from, "r", encoding="utf-8") as infile:
    for line in infile:
      line = line.strip()
      if line.startswith("VERSION = "):
        version = line.split("=", 1)[1].strip().strip("'\"")

  if version is None:
    raise ValueError(
        "File {} does not container a 'VERSION = '"
        .format(args.version_from))

  attrs = {
      "name": args.name,
      "version": version
  }

  if args.has_extension_module:
    attrs["ext_modules"] = [
        Extension("__dummy", ["dummy.c"])
    ]

  dist = Distribution(attrs=attrs)
  bdist_wheel_cmd = dist.get_command_obj('bdist_wheel')
  bdist_wheel_cmd.ensure_finalized()
  distname = bdist_wheel_cmd.wheel_dist_name
  tag = '-'.join(bdist_wheel_cmd.get_tag())

  print("{}-{}.whl".format(distname, tag))


if __name__ == "__main__":
  main()



# pylint: disable=bad-continuation
# pylint: disable=too-many-lines

# NOTE(josh): These are automatically generated using `cmake-genparsers` on
# the distribution provided standard modules in cmake version 3.10
FUNSPECS = {
  "add_command": {
    "pargs": {
      "nargs": 1
    }
  },
  "add_compiler_export_flags": {
    "pargs": {
      "nargs": 0
    }
  },
  "add_feature_info": {
    "pargs": {
      "nargs": 3
    }
  },
  "add_file_dependencies": {
    "pargs": {
      "nargs": 1
    }
  },
  "add_flex_bison_dependency": {
    "pargs": {
      "nargs": 2
    }
  },
  "add_jar": {
    "kwargs": {
      "ENTRY_POINT": 1,
      "INCLUDE_JARS": "+",
      "MANIFEST": 1,
      "OUTPUT_DIR": 1,
      "OUTPUT_NAME": 1,
      "SOURCES": "+",
      "VERSION": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "adjust_cmake_system_variables": {
    "pargs": {
      "nargs": 1
    }
  },
  "android_add_test_data": {
    "kwargs": {
      "DEVICE_OBJECT_STORE": 1,
      "DEVICE_TEST_DIR": 1,
      "FILES": "+",
      "FILES_DEST": 1,
      "LIBS": "+",
      "LIBS_DEST": 1,
      "NO_LINK_REGEX": "+"
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "bison_target": {
    "kwargs": {
      "COMPILE_FLAGS": 1,
      "DEFINES_FILE": 1,
      "REPORT_FILE": 1,
      "VERBOSE": "+"
    },
    "pargs": {

      "nargs": "3+"
    }
  },
  "bison_target_option_defines": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "BISON_TARGET_option_defines"
  },
  "bison_target_option_extraopts": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "BISON_TARGET_option_extraopts"
  },
  "bison_target_option_report_file": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "BISON_TARGET_option_report_file"
  },
  "bison_target_option_verbose": {
    "pargs": {
      "nargs": 3
    },
    "spelling": "BISON_TARGET_option_verbose"
  },
  "check_c_compiler_flag": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_c_source_compiles": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_c_source_runs": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_compiler_flag_common_patterns": {
    "pargs": {
      "nargs": 1
    }
  },
  "check_cxx_accepts_flag": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_cxx_compiler_flag": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_cxx_source_compiles": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_cxx_source_runs": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_cxx_symbol_exists": {
    "pargs": {
      "nargs": 3
    }
  },
  "check_fortran_compiler_flag": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "CHECK_Fortran_COMPILER_FLAG"
  },
  "check_fortran_function_exists": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_fortran_libraries": {
    "pargs": {
      "nargs": 6
    },
    "spelling": "Check_Fortran_Libraries"
  },
  "check_fortran_source_compiles": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "CHECK_Fortran_SOURCE_COMPILES"
  },
  "check_function_exists": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_include_file": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_include_file_cxx": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_include_files": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_ipo_supported": {
    "kwargs": {
      "LANGUAGES": "+",
      "OUTPUT": 1,
      "RESULT": 1
    },
    "pargs": {

      "nargs": "*"
    }
  },
  "check_language": {
    "pargs": {
      "nargs": 1
    }
  },
  "check_lapack_libraries": {
    "pargs": {
      "nargs": 7
    },
    "spelling": "Check_Lapack_Libraries"
  },
  "check_library_exists": {
    "pargs": {
      "nargs": 4
    }
  },
  "check_prototype_definition": {
    "pargs": {
      "nargs": 5
    }
  },
  "check_required_var": {
    "pargs": {
      "nargs": 1
    }
  },
  "check_struct_has_member": {
    "pargs": {
      "nargs": 4
    }
  },
  "check_symbol_exists": {
    "pargs": {
      "nargs": 3
    }
  },
  "check_type_size": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_variable_exists": {
    "pargs": {
      "nargs": 2
    }
  },
  "check_version": {
    "pargs": {
      "nargs": 2
    }
  },
  "clear_bundle_keys": {
    "pargs": {
      "nargs": 1
    }
  },
  "cmake_add_fortran_subdirectory": {
    "kwargs": {
      "ARCHIVE_DIR": 1,
      "CMAKE_COMMAND_LINE": "+",
      "LIBRARIES": "+",
      "LINK_LIBRARIES": "+",
      "PROJECT": 1,
      "RUNTIME_DIR": 1
    },
    "pargs": {
      "flags": [
        "NO_EXTERNAL_INSTALL"
      ],
      "nargs": "1+"
    }
  },
  "cmake_dependent_option": {
    "pargs": {
      "nargs": 5
    }
  },
  "cmake_determine_compile_features": {
    "pargs": {
      "nargs": 1
    }
  },
  "cmake_determine_compiler_abi": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_determine_compiler_id": {
    "pargs": {
      "nargs": 3
    }
  },
  "cmake_determine_compiler_id_build": {
    "pargs": {
      "nargs": 4
    }
  },
  "cmake_determine_compiler_id_check": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_determine_compiler_id_match_vendor": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_determine_compiler_id_vendor": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_determine_compiler_id_write": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_determine_msvc_showincludes_prefix": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_diagnose_unsupported_clang": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_expand_imported_targets": {
    "kwargs": {
      "CONFIGURATION": 1,
      "LIBRARIES": "+"
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "cmake_export_build_settings": {
    "pargs": {
      "nargs": 1
    }
  },
  "cmake_find_frameworks": {
    "pargs": {
      "nargs": 1
    }
  },
  "cmake_force_c_compiler": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_force_cxx_compiler": {
    "pargs": {
      "nargs": 2
    }
  },
  "cmake_force_fortran_compiler": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "CMAKE_FORCE_Fortran_COMPILER"
  },
  "cmake_import_build_settings": {
    "pargs": {
      "nargs": 1
    }
  },
  "cmake_parse_implicit_link_info": {
    "pargs": {
      "nargs": 6
    }
  },
  "cmake_pop_check_state": {
    "pargs": {
      "nargs": 0
    }
  },
  "cmake_print_properties": {
    "kwargs": {
      "CACHE_ENTRIES": "+",
      "DIRECTORIES": "+",
      "PROPERTIES": "+",
      "SOURCES": "+",
      "TARGETS": "+",
      "TESTS": "+"
    },
    "pargs": {

      "nargs": "*"
    }
  },
  "cmake_print_variables": {
    "pargs": {
      "nargs": 0
    }
  },
  "cmake_push_check_state": {
    "pargs": {
      "nargs": 0
    }
  },
  "cmake_reset_check_state": {
    "pargs": {
      "nargs": 0
    }
  },
  "compiler_id_detection": {
    "kwargs": {
      "PREFIX": 1
    },
    "pargs": {
      "flags": [
        "ID_STRING",
        "VERSION_STRINGS",
        "ID_DEFINE",
        "PLATFORM_DEFAULT_COMPILER"
      ],
      "nargs": "2+"
    }
  },
  "configure_package_config_file": {
    "kwargs": {
      "INSTALL_DESTINATION": 1,
      "INSTALL_PREFIX": 1,
      "PATH_VARS": "+"
    },
    "pargs": {
      "flags": [
        "NO_SET_AND_CHECK_MACRO",
        "NO_CHECK_REQUIRED_COMPONENTS_MACRO"
      ],
      "nargs": "2+"
    }
  },
  "copy_and_fixup_bundle": {
    "pargs": {
      "nargs": 4
    }
  },
  "copy_resolved_framework_into_bundle": {
    "pargs": {
      "nargs": 2
    }
  },
  "copy_resolved_item_into_bundle": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_add_component": {
    "kwargs": {
      "ARCHIVE_FILE": 1,
      "DEPENDS": "+",
      "DESCRIPTION": 1,
      "DISPLAY_NAME": 1,
      "GROUP": 1,
      "INSTALL_TYPES": "+",
      "PLIST": 1
    },
    "pargs": {
      "flags": [
        "HIDDEN",
        "REQUIRED",
        "DISABLED",
        "DOWNLOADED"
      ],
      "nargs": "1+"
    }
  },
  "cpack_add_component_group": {
    "kwargs": {
      "DESCRIPTION": 1,
      "DISPLAY_NAME": 1,
      "PARENT_GROUP": 1
    },
    "pargs": {
      "flags": [
        "EXPANDED",
        "BOLD_TITLE"
      ],
      "nargs": "1+"
    }
  },
  "cpack_add_install_type": {
    "kwargs": {
      "DISPLAY_NAME": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "cpack_append_list_variable_set_command": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_append_option_set_command": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_append_string_variable_set_command": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_append_variable_set_command": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_check_file_exists": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_configure_downloads": {
    "kwargs": {
      "UPLOAD_DIRECTORY": 1
    },
    "pargs": {
      "flags": [
        "ALL",
        "ADD_REMOVE",
        "NO_ADD_REMOVE"
      ],
      "nargs": "1+"
    }
  },
  "cpack_deb_prepare_package_vars": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_deb_variable_fallback": {
    "pargs": {
      "nargs": 1
    }
  },
  "cpack_encode_variables": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_ifw_add_package_resources": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_ifw_add_repository": {
    "kwargs": {
      "DISPLAY_NAME": 1,
      "PASSWORD": 1,
      "URL": 1,
      "USERNAME": 1
    },
    "pargs": {
      "flags": [
        "DISABLED"
      ],
      "nargs": "1+"
    }
  },
  "cpack_ifw_configure_component": {
    "kwargs": {
      "AUTO_DEPEND_ON": "+",
      "CHECKABLE": 1,
      "DEFAULT": 1,
      "DEPENDENCIES": "+",
      "DEPENDS": "+",
      "DESCRIPTION": "+",
      "DISPLAY_NAME": "+",
      "LICENSES": "+",
      "NAME": 1,
      "PRIORITY": 1,
      "RELEASE_DATE": 1,
      "REPLACES": "+",
      "SCRIPT": 1,
      "SORTING_PRIORITY": 1,
      "TRANSLATIONS": "+",
      "UPDATE_TEXT": 1,
      "USER_INTERFACES": "+",
      "VERSION": 1
    },
    "pargs": {
      "flags": [
        "COMMON",
        "ESSENTIAL",
        "VIRTUAL",
        "FORCED_INSTALLATION",
        "REQUIRES_ADMIN_RIGHTS"
      ],
      "nargs": "1+"
    }
  },
  "cpack_ifw_configure_component_group": {
    "kwargs": {
      "AUTO_DEPEND_ON": "+",
      "CHECKABLE": 1,
      "DEFAULT": 1,
      "DEPENDENCIES": "+",
      "DEPENDS": "+",
      "DESCRIPTION": "+",
      "DISPLAY_NAME": "+",
      "LICENSES": "+",
      "NAME": 1,
      "PRIORITY": 1,
      "RELEASE_DATE": 1,
      "REPLACES": "+",
      "SCRIPT": 1,
      "SORTING_PRIORITY": 1,
      "TRANSLATIONS": "+",
      "UPDATE_TEXT": 1,
      "USER_INTERFACES": "+",
      "VERSION": 1
    },
    "pargs": {
      "flags": [
        "VIRTUAL",
        "FORCED_INSTALLATION",
        "REQUIRES_ADMIN_RIGHTS"
      ],
      "nargs": "1+"
    }
  },
  "cpack_ifw_configure_file": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_ifw_update_repository": {
    "kwargs": {
      "DISPLAY_NAME": 1,
      "NEW_URL": 1,
      "OLD_URL": 1,
      "PASSWORD": 1,
      "URL": 1,
      "USERNAME": 1
    },
    "pargs": {
      "flags": [
        "ADD",
        "REMOVE",
        "REPLACE",
        "DISABLED"
      ],
      "nargs": "1+"
    }
  },
  "cpack_optional_append": {
    "pargs": {
      "nargs": 3
    }
  },
  "cpack_rpm_debugsymbol_check": {
    "pargs": {
      "nargs": 2
    }
  },
  "cpack_rpm_generate_package": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_rpm_prepare_content_list": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_rpm_prepare_install_files": {
    "pargs": {
      "nargs": 4
    }
  },
  "cpack_rpm_prepare_relocation_paths": {
    "pargs": {
      "nargs": 0
    }
  },
  "cpack_rpm_symlink_add_for_relocation_script": {
    "pargs": {
      "nargs": 5
    }
  },
  "cpack_rpm_symlink_create_relocation_script": {
    "pargs": {
      "nargs": 1
    }
  },
  "cpack_rpm_symlink_get_relocation_prefixes": {
    "pargs": {
      "nargs": 3
    }
  },
  "cpack_rpm_variable_fallback": {
    "pargs": {
      "nargs": 1
    }
  },
  "cpack_set_if_not_set": {
    "pargs": {
      "nargs": 2
    }
  },
  "create_javadoc": {
    "pargs": {
      "nargs": 1
    }
  },
  "create_javah": {
    "kwargs": {
      "CLASSES": "+",
      "CLASSPATH": "+",
      "DEPENDS": "+",
      "GENERATED_FILES": 1,
      "OUTPUT_DIR": 1,
      "OUTPUT_NAME": 1,
      "TARGET": 1
    },
    "pargs": {

      "nargs": "*"
    }
  },
  "crt_version": {
    "pargs": {
      "nargs": 2
    }
  },
  "csharp_get_dependentupon_name": {
    "pargs": {
      "nargs": 2
    }
  },
  "csharp_get_filename_key_base": {
    "pargs": {
      "nargs": 2
    }
  },
  "csharp_get_filename_keys": {
    "pargs": {
      "nargs": 1
    }
  },
  "csharp_set_designer_cs_properties": {
    "pargs": {
      "nargs": 0
    }
  },
  "csharp_set_windows_forms_properties": {
    "pargs": {
      "nargs": 0
    }
  },
  "csharp_set_xaml_cs_properties": {
    "pargs": {
      "nargs": 0
    }
  },
  "ctest_coverage_collect_gcov": {
    "kwargs": {
      "BUILD": 1,
      "GCOV_COMMAND": 1,
      "GCOV_OPTIONS": "+",
      "SOURCE": 1,
      "TARBALL": 1
    },
    "pargs": {
      "flags": [
        "QUIET",
        "GLOB",
        "DELETE"
      ],
      "nargs": "*"
    }
  },
  "cuda_add_cublas_to_target": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_add_cuda_include_once": {
    "pargs": {
      "nargs": 0
    }
  },
  "cuda_add_cufft_to_target": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_add_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_add_library": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_build_clean_target": {
    "pargs": {
      "nargs": 0
    }
  },
  "cuda_build_shared_library": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_compile": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_compile_base": {
    "pargs": {
      "nargs": 3
    }
  },
  "cuda_compile_cubin": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_compile_fatbin": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_compile_ptx": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_compute_build_path": {
    "pargs": {
      "nargs": 2
    }
  },
  "cuda_compute_separable_compilation_object_file_name": {
    "pargs": {
      "nargs": 3
    }
  },
  "cuda_find_helper_file": {
    "pargs": {
      "nargs": 2
    }
  },
  "cuda_find_host_program": {
    "pargs": {
      "nargs": 0
    }
  },
  "cuda_find_library_local_first": {
    "pargs": {
      "nargs": 3
    }
  },
  "cuda_find_library_local_first_with_path_ext": {
    "pargs": {
      "nargs": 4
    }
  },
  "cuda_get_sources_and_options": {
    "pargs": {
      "nargs": 3
    }
  },
  "cuda_include_directories": {
    "pargs": {
      "nargs": 0
    }
  },
  "cuda_include_nvcc_dependencies": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_link_separable_compilation_objects": {
    "pargs": {
      "nargs": 4
    }
  },
  "cuda_parse_nvcc_options": {
    "pargs": {
      "nargs": 1
    }
  },
  "cuda_unset_include_and_libraries": {
    "pargs": {
      "nargs": 0
    }
  },
  "cuda_wrap_srcs": {
    "pargs": {
      "nargs": 3
    }
  },
  "cxxtest_add_test": {
    "pargs": {
      "nargs": 2
    }
  },
  "dbg_msg": {
    "pargs": {
      "nargs": 1
    }
  },
  "dbg_msg_v": {
    "pargs": {
      "nargs": 1
    }
  },
  "determinevsservicepack": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "DetermineVSServicePack"
  },
  "doxygen_add_docs": {
    "kwargs": {
      "COMMENT": 1,
      "WORKING_DIRECTORY": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "doxygen_list_to_quoted_strings": {
    "pargs": {
      "nargs": 1
    }
  },
  "doxygen_quote_value": {
    "pargs": {
      "nargs": 1
    }
  },
  "dump_file": {
    "pargs": {
      "nargs": 1
    }
  },
  "ecos_add_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "ecos_add_include_directories": {
    "pargs": {
      "nargs": 0
    }
  },
  "ecos_add_target_lib": {
    "pargs": {
      "nargs": 0
    }
  },
  "ecos_adjust_directory": {
    "pargs": {
      "nargs": 1
    }
  },
  "ecos_use_arm_elf_tools": {
    "pargs": {
      "nargs": 0
    }
  },
  "ecos_use_i386_elf_tools": {
    "pargs": {
      "nargs": 0
    }
  },
  "ecos_use_ppc_eabi_tools": {
    "pargs": {
      "nargs": 0
    }
  },
  "enable_language": {
    "pargs": {
      "nargs": 0
    }
  },
  "export_jars": {
    "kwargs": {
      "FILE": 1,
      "NAMESPACE": 1,
      "TARGETS": "+"
    },
    "pargs": {

      "nargs": "*"
    }
  },
  "externaldata_add_target": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "ExternalData_add_target"
  },
  "externaldata_add_test": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "ExternalData_add_test"
  },
  "externaldata_expand_arguments": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "ExternalData_expand_arguments"
  },
  "extract_matlab_versions_from_registry_brute_force": {
    "pargs": {
      "nargs": 1
    }
  },
  "extract_so_info": {
    "pargs": {
      "nargs": 3
    }
  },
  "feature_summary": {
    "kwargs": {
      "DESCRIPTION": 1,
      "FILENAME": 1,
      "VAR": 1,
      "WHAT": "+"
    },
    "pargs": {
      "flags": [
        "APPEND",
        "INCLUDE_QUIET_PACKAGES",
        "FATAL_ON_MISSING_REQUIRED_PACKAGES",
        "QUIET_ON_EMPTY",
        "DEFAULT_DESCRIPTION"
      ],
      "nargs": "*"
    }
  },
  "find_cuda_helper_libs": {
    "pargs": {
      "nargs": 1
    }
  },
  "find_dependency": {
    "pargs": {
      "nargs": 1
    }
  },
  "find_imagemagick_api": {
    "pargs": {
      "nargs": 2
    }
  },
  "find_imagemagick_exe": {
    "pargs": {
      "nargs": 1
    }
  },
  "find_jar": {
    "pargs": {
      "nargs": 1
    }
  },
  "find_library_local_first": {
    "pargs": {
      "nargs": 3
    }
  },
  "find_package_handle_standard_args": {
    "kwargs": {
      "FAIL_MESSAGE": 1,
      "FOUND_VAR": 1,
      "REQUIRED_VARS": "+",
      "VERSION_VAR": 1
    },
    "pargs": {
      "flags": [
        "CONFIG_MODE",
        "HANDLE_COMPONENTS"
      ],
      "nargs": "2+"
    }
  },
  "find_package_message": {
    "pargs": {
      "nargs": 3
    }
  },
  "fixup_bundle": {
    "kwargs": {
      "IGNORE_ITEM": "+"
    },
    "pargs": {

      "nargs": "3+"
    }
  },
  "fixup_bundle_item": {
    "pargs": {
      "nargs": 3
    }
  },
  "fixup_qt4_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "flex_target": {
    "kwargs": {
      "COMPILE_FLAGS": 1,
      "DEFINES_FILE": 1
    },
    "pargs": {

      "nargs": "3+"
    }
  },
  "fortrancinterface_header": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "FortranCInterface_HEADER"
  },
  "fortrancinterface_verify": {
    "pargs": {
      "nargs": 0
    },
    "spelling": "FortranCInterface_VERIFY"
  },
  "from_hex": {
    "pargs": {
      "nargs": 2
    }
  },
  "generate_export_header": {
    "pargs": {
      "nargs": 1
    }
  },
  "get_bundle_all_executables": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_bundle_and_executable": {
    "pargs": {
      "nargs": 4
    }
  },
  "get_bundle_keys": {
    "kwargs": {
      "IGNORE_ITEM": "+"
    },
    "pargs": {

      "nargs": "4+"
    }
  },
  "get_bundle_main_executable": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_component_package_name": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_dotapp_dir": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_item_key": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_item_rpaths": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_prerequisites": {
    "pargs": {
      "nargs": 6
    }
  },
  "get_unix_permissions_octal_notation": {
    "pargs": {
      "nargs": 2
    }
  },
  "get_vs_version_string": {
    "pargs": {
      "nargs": 2
    }
  },
  "getdefaultwindowsprefixbase": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "GetDefaultWindowsPrefixBase"
  },
  "gettext_create_translations": {
    "pargs": {
      "nargs": 2
    }
  },
  "gettext_process_po_files": {
    "kwargs": {
      "INSTALL_DESTINATION": 1,
      "PO_FILES": "+"
    },
    "pargs": {
      "flags": [
        "ALL"
      ],
      "nargs": "1+"
    }
  },
  "gettext_process_pot_file": {
    "kwargs": {
      "INSTALL_DESTINATION": 1,
      "LANGUAGES": "+"
    },
    "pargs": {
      "flags": [
        "ALL"
      ],
      "nargs": "1+"
    }
  },
  "gnuinstalldirs_get_absolute_install_dir": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "GNUInstallDirs_get_absolute_install_dir"
  },
  "gp_append_unique": {
    "pargs": {
      "nargs": 2
    }
  },
  "gp_file_type": {
    "pargs": {
      "nargs": 3
    }
  },
  "gp_item_default_embedded_path": {
    "pargs": {
      "nargs": 2
    }
  },
  "gp_resolve_item": {
    "pargs": {
      "nargs": 5
    }
  },
  "gp_resolved_file_type": {
    "pargs": {
      "nargs": 5
    }
  },
  "gtest_add_tests": {
    "kwargs": {
      "EXTRA_ARGS": "+",
      "SOURCES": "+",
      "TARGET": 1,
      "TEST_LIST": 1,
      "TEST_PREFIX": 1,
      "TEST_SUFFIX": 1,
      "WORKING_DIRECTORY": 1
    },
    "pargs": {
      "flags": [
        "SKIP_DEPENDENCY"
      ],
      "nargs": "*"
    }
  },
  "gtest_discover_tests": {
    "kwargs": {
      "EXTRA_ARGS": "+",
      "PROPERTIES": "+",
      "TEST_LIST": 1,
      "TEST_PREFIX": 1,
      "TEST_SUFFIX": 1,
      "TIMEOUT": 1,
      "WORKING_DIRECTORY": 1
    },
    "pargs": {
      "flags": [
        "NO_PRETTY_TYPES",
        "NO_PRETTY_VALUES"
      ],
      "nargs": "1+"
    }
  },
  "hg_wc_info": {
    "pargs": {
      "nargs": 2
    }
  },
  "install_jar": {
    "kwargs": {
      "COMPONENT": 1,
      "DESTINATION": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "install_jar_exports": {
    "kwargs": {
      "COMPONENT": 1,
      "DESTINATION": 1,
      "FILE": 1,
      "NAMESPACE": 1,
      "TARGETS": "+"
    },
    "pargs": {

      "nargs": "*"
    }
  },
  "install_jni_symlink": {
    "kwargs": {
      "COMPONENT": 1,
      "DESTINATION": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "install_qt4_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "install_qt4_plugin": {
    "pargs": {
      "nargs": 4
    }
  },
  "install_qt4_plugin_path": {
    "pargs": {
      "nargs": 4
    }
  },
  "ios_install_combined": {
    "pargs": {
      "nargs": 2
    }
  },
  "is_file_executable": {
    "pargs": {
      "nargs": 2
    }
  },
  "java_append_library_directories": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_dcop_skels": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_dcop_stubs": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_kcfg_files": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_kdeinit_executable": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_kpart": {
    "pargs": {
      "nargs": 2
    }
  },
  "kde3_add_moc_files": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_add_ui_files": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_automoc": {
    "pargs": {
      "nargs": 0
    }
  },
  "kde3_create_final_file": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_install_icons": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_install_libtool_file": {
    "pargs": {
      "nargs": 1
    }
  },
  "kde3_print_results": {
    "pargs": {
      "nargs": 0
    }
  },
  "link_resolved_item_into_bundle": {
    "pargs": {
      "nargs": 2
    }
  },
  "list_prerequisites": {
    "pargs": {
      "nargs": 1
    }
  },
  "list_prerequisites_by_glob": {
    "pargs": {
      "nargs": 2
    }
  },
  "macro_add_file_dependencies": {
    "pargs": {
      "nargs": 1
    }
  },
  "matlab_add_mex": {
    "kwargs": {
      "DOCUMENTATION": 1,
      "LINK_TO": "+",
      "NAME": 1,
      "OUTPUT_NAME": 1,
      "SRC": "+"
    },
    "pargs": {
      "flags": [
        "EXECUTABLE",
        "MODULE",
        "SHARED"
      ],
      "nargs": "*"
    }
  },
  "matlab_add_unit_test": {
    "kwargs": {
      "NO_UNITTEST_FRAMEWORK": "+",
      "_MATLAB_UNITTEST_PREFIX": 1
    },
    "pargs": {
      "flags": [
        "0"
      ],
      "nargs": "*"
    }
  },
  "matlab_extract_all_installed_versions_from_registry": {
    "pargs": {
      "nargs": 2
    }
  },
  "matlab_get_all_valid_matlab_roots_from_registry": {
    "pargs": {
      "nargs": 2
    }
  },
  "matlab_get_mex_suffix": {
    "pargs": {
      "nargs": 2
    }
  },
  "matlab_get_release_name_from_version": {
    "pargs": {
      "nargs": 2
    }
  },
  "matlab_get_supported_releases": {
    "pargs": {
      "nargs": 1
    }
  },
  "matlab_get_supported_versions": {
    "pargs": {
      "nargs": 1
    }
  },
  "matlab_get_version_from_matlab_run": {
    "pargs": {
      "nargs": 2
    }
  },
  "matlab_get_version_from_release_name": {
    "pargs": {
      "nargs": 2
    }
  },
  "mpi_search_mpi_prefix_folder": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "MPI_search_mpi_prefix_folder"
  },
  "msg": {
    "pargs": {
      "nargs": 1
    }
  },
  "osg_find_library": {
    "pargs": {
      "nargs": 2
    }
  },
  "osg_find_path": {
    "pargs": {
      "nargs": 2
    }
  },
  "osg_mark_as_advanced": {
    "pargs": {
      "nargs": 1
    }
  },
  "perl_adjust_darwin_lib_variable": {
    "pargs": {
      "nargs": 1
    }
  },
  "pkg_check_modules": {
    "pargs": {
      "nargs": 2
    }
  },
  "pkg_get_variable": {
    "pargs": {
      "nargs": 3
    }
  },
  "pkg_search_module": {
    "pargs": {
      "nargs": 2
    }
  },
  "pkgconfig": {
    "pargs": {
      "nargs": 5
    }
  },
  "print_disabled_features": {
    "pargs": {
      "nargs": 0
    }
  },
  "print_enabled_features": {
    "pargs": {
      "nargs": 0
    }
  },
  "printtestcompilerstatus": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "PrintTestCompilerStatus"
  },
  "processorcount": {
    "pargs": {
      "nargs": 1
    },
    "spelling": "ProcessorCount"
  },
  "protobuf_generate_cpp": {
    "kwargs": {
      "DESCRIPTORS": 1,
      "EXPORT_MACRO": 1
    },
    "pargs": {

      "nargs": "2+"
    }
  },
  "protobuf_generate_python": {
    "pargs": {
      "nargs": 1
    }
  },
  "python_add_module": {
    "pargs": {
      "nargs": 1
    }
  },
  "python_write_modules_header": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_add_dbus_adaptor": {
    "pargs": {
      "nargs": 4
    }
  },
  "qt4_add_dbus_interface": {
    "pargs": {
      "nargs": 3
    }
  },
  "qt4_add_dbus_interfaces": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_add_resources": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_add_translation": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_automoc": {
    "pargs": {
      "nargs": 0
    }
  },
  "qt4_create_moc_command": {
    "pargs": {
      "nargs": 5
    }
  },
  "qt4_create_translation": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_extract_options": {
    "pargs": {
      "nargs": 3
    }
  },
  "qt4_generate_dbus_interface": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_generate_moc": {
    "pargs": {
      "nargs": 2
    }
  },
  "qt4_get_moc_flags": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_make_output_file": {
    "pargs": {
      "nargs": 4
    }
  },
  "qt4_use_modules": {
    "pargs": {
      "nargs": 2
    }
  },
  "qt4_wrap_cpp": {
    "pargs": {
      "nargs": 1
    }
  },
  "qt4_wrap_ui": {
    "pargs": {
      "nargs": 1
    }
  },
  "resolve_qt4_paths": {
    "pargs": {
      "nargs": 1
    }
  },
  "rti_message_quietly": {
    "pargs": {
      "nargs": 3
    }
  },
  "select_library_configurations": {
    "pargs": {
      "nargs": 1
    }
  },
  "set_bundle_key_values": {
    "pargs": {
      "nargs": 6
    }
  },
  "set_compile_flags_var": {
    "pargs": {
      "nargs": 1
    }
  },
  "set_feature_info": {
    "pargs": {
      "nargs": 0
    }
  },
  "set_if_not_set": {
    "pargs": {
      "nargs": 2
    }
  },
  "set_if_set": {
    "pargs": {
      "nargs": 2
    }
  },
  "set_if_set_and_not_set": {
    "pargs": {
      "nargs": 2
    }
  },
  "set_link_flags_var": {
    "pargs": {
      "nargs": 1
    }
  },
  "set_package_info": {
    "pargs": {
      "nargs": 2
    }
  },
  "set_package_properties": {
    "kwargs": {
      "DESCRIPTION": 1,
      "PURPOSE": 1,
      "TYPE": 1,
      "URL": 1
    },
    "pargs": {

      "nargs": "2+"
    }
  },
  "source_files": {
    "pargs": {
      "nargs": 0
    }
  },
  "squish_add_test": {
    "pargs": {
      "nargs": 0
    }
  },
  "squish_v3_add_test": {
    "pargs": {
      "nargs": 5
    }
  },
  "squish_v4_add_test": {
    "kwargs": {
      "AUT": 1,
      "POST_COMMAND": 1,
      "PRE_COMMAND": 1,
      "SETTINGSGROUP": 1,
      "SUITE": 1,
      "TEST": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "subversion_wc_info": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "Subversion_WC_INFO"
  },
  "subversion_wc_log": {
    "pargs": {
      "nargs": 2
    },
    "spelling": "Subversion_WC_LOG"
  },
  "swig_add_library": {
    "kwargs": {
      "LANGUAGE": 1,
      "SOURCES": "+",
      "TYPE": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "swig_add_module": {
    "pargs": {
      "nargs": 2
    }
  },
  "swig_add_source_to_module": {
    "pargs": {
      "nargs": 3
    }
  },
  "swig_get_extra_output_files": {
    "pargs": {
      "nargs": 4
    }
  },
  "swig_link_libraries": {
    "pargs": {
      "nargs": 1
    }
  },
  "swig_module_initialize": {
    "pargs": {
      "nargs": 2
    }
  },
  "test_big_endian": {
    "pargs": {
      "nargs": 1
    }
  },
  "verify_app": {
    "kwargs": {
      "IGNORE_ITEM": "+"
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "verify_bundle_prerequisites": {
    "kwargs": {
      "IGNORE_ITEM": "+"
    },
    "pargs": {

      "nargs": "3+"
    }
  },
  "verify_bundle_symlinks": {
    "pargs": {
      "nargs": 3
    }
  },
  "write_basic_config_version_file": {
    "kwargs": {
      "COMPATIBILITY": 1,
      "VERSION": 1
    },
    "pargs": {

      "nargs": "1+"
    }
  },
  "write_basic_package_version_file": {
    "pargs": {
      "nargs": 0
    },
    "kwargs": {
      "COMPATIBILITY": 1,
      "VERSION": 1
    }
  },
  "write_compiler_detection_header": {
    "kwargs": {
      "COMPILERS": "+",
      "EPILOG": 1,
      "FEATURES": "+",
      "OUTPUT_DIR": 1,
      "OUTPUT_FILES_VAR": 1,
      "PROLOG": 1,
      "VERSION": 1
    },
    "pargs": {
      "flags": [
        "ALLOW_UNKNOWN_COMPILERS",
        "ALLOW_UNKNOWN_COMPILER_VERSIONS"
      ],
      "nargs": "4+"
    }
  },
  "write_qt4_conf": {
    "pargs": {
      "nargs": 2
    }
  },
  "wx_clear_all_dbg_libs": {
    "pargs": {
      "nargs": 0
    }
  },
  "wx_clear_all_libs": {
    "pargs": {
      "nargs": 1
    }
  },
  "wx_clear_all_rel_libs": {
    "pargs": {
      "nargs": 0
    }
  },
  "wx_clear_lib": {
    "pargs": {
      "nargs": 1
    }
  },
  "wx_config_select_get_default": {
    "pargs": {
      "nargs": 0
    }
  },
  "wx_config_select_query_bool": {
    "pargs": {
      "nargs": 2
    }
  },
  "wx_config_select_set_options": {
    "pargs": {
      "nargs": 0
    }
  },
  "wx_find_libs": {
    "pargs": {
      "nargs": 3
    }
  },
  "wx_get_dependencies_from_xml": {
    "pargs": {
      "nargs": 5
    }
  },
  "wx_get_name_components": {
    "pargs": {
      "nargs": 4
    }
  },
  "wx_set_libraries": {
    "pargs": {
      "nargs": 2
    }
  },
  "wx_split_arguments_on": {
    "pargs": {
      "nargs": 3
    }
  },
  "wxwidgets_add_resources": {
    "pargs": {
      "nargs": 1
    }
  },
  "xctest_add_bundle": {
    "pargs": {
      "nargs": 2
    }
  },
  "xctest_add_test": {
    "pargs": {
      "nargs": 2
    }
  }
}



# pylint: disable=invalid-name
# Additional FLAGS and KWARGS for custom commands
additional_commands = {
    "foo": {
        "flags": ["BAR", "BAZ"],
        "kwargs": {
            "HEADERS": '*',
            "SOURCES": '*',
            "DEPENDS": '*',
        }
    }
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import contextlib
import logging
import unittest
import sys

from cmakelang.format import __main__
from cmakelang import configuration
from cmakelang import lex
from cmakelang import parse
from cmakelang.format import formatter
from cmakelang.parse.common import NodeType


def strip_indent(content, indent=6):
  """
  Strings used in this file are indented by 6-spaces to keep them readable
  within the python code that they are embedded. Remove those 6-spaces from
  the front of each line before running the tests.
  """

  # NOTE(josh): don't use splitlines() so that we get the same result
  # regardless of windows or unix line endings in content.
  return '\n'.join([line[indent:] for line in content.split('\n')])


def overzip(iterable_a, iterable_b):
  """
  Like itertools.izip but instead if the two lists have different sizes then
  the resulting generator will yield a number of pairs equal to the larger of
  the two inputs (rathe than the smaller). The empty list will be padded with
  None elements.
  """
  iter_a = iter(iterable_a)
  iter_b = iter(iterable_b)

  item_a = next(iter_a, None)
  item_b = next(iter_b, None)

  while item_a is not None and item_b is not None:
    yield(item_a, item_b)
    item_a = next(iter_a, None)
    item_b = next(iter_b, None)

  while item_a is not None:
    yield(item_a, None)
    item_a = next(iter_a, None)

  while item_b is not None:
    yield(None, item_b)
    item_b = next(iter_b, None)


def assert_tree(test, nodes, tups, tree=None, history=None):
  if tree is None:
    tree = nodes
  if history is None:
    history = []

  for node, tup in overzip(nodes, tups):
    if isinstance(node, lex.Token):
      continue
    subhistory = history + [node]
    message = (" for node {} at\n {} \n\n\n"
               "If the actual result is expected, then update the test with"
               " this:\n# pylint: disable=bad-continuation\n# noqa: E122\n{}"
               .format(node,
                       formatter.tree_string(tree, subhistory),
                       formatter.test_string(tree)))
    test.assertIsNotNone(node, msg="Missing node" + message)
    test.assertIsNotNone(tup, msg="Extra node" + message)
    if len(tup) == 6:
      ntype, wrap, row, col, colextent, expect_children = tup
      test.assertEqual(node.passno, wrap,
                       msg="Expected wrap={}".format(wrap) + message)
    else:
      ntype, row, col, colextent, expect_children = tup

    test.assertEqual(node.node_type, ntype,
                     msg="Expected type={}".format(ntype) + message)
    test.assertEqual(node.position[0], row,
                     msg="Expected row={}".format(row) + message)
    test.assertEqual(node.position[1], col,
                     msg="Expected col={}".format(col) + message)
    test.assertEqual(node.colextent, colextent,
                     msg="Expected colextent={}".format(colextent) + message)
    assert_tree(test, node.children, expect_children, tree, subhistory)


class TestCanonicalLayout(unittest.TestCase):
  """
  Given a bunch of example inputs, ensure that the output is as expected.
  """

  def __init__(self, *args, **kwargs):
    super(TestCanonicalLayout, self).__init__(*args, **kwargs)
    self.config = configuration.Configuration()
    parse_db = parse.funs.get_parse_db()
    self.parse_ctx = parse.ParseContext(parse_db)

  def setUp(self):
    self.config.parse.fn_spec.add(
        'foo',
        flags=['BAR', 'BAZ'],
        kwargs={
            "HEADERS": '*',
            "SOURCES": '*',
            "DEPENDS": '*'
        })

    self.parse_ctx.parse_db.update(
        parse.funs.get_funtree(self.config.parse.fn_spec))

  def tearDown(self):
    pass

  @contextlib.contextmanager
  def subTest(self, msg=None, **params):
    # pylint: disable=no-member
    if sys.version_info < (3, 4, 0):
      yield None
    else:
      yield super(TestCanonicalLayout, self).subTest(msg=msg, **params)

  def do_layout_test(self, input_str, expect_tree, strip_len=6):
    """
    Run the formatter on the input string and assert that the result matches
    the output string
    """

    input_str = strip_indent(input_str, strip_len)
    tokens = lex.tokenize(input_str)
    parse_tree = parse.parse(tokens, self.parse_ctx)
    box_tree = formatter.layout_tree(parse_tree, self.config)
    assert_tree(self, [box_tree], expect_tree)

  def test_simple_statement(self):
    self.do_layout_test("""\
      cmake_minimum_required(VERSION 2.8.11)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 38, [
  (NodeType.STATEMENT, 0, 0, 0, 38, [
    (NodeType.FUNNAME, 0, 0, 0, 22, []),
    (NodeType.LPAREN, 0, 0, 22, 23, []),
    (NodeType.ARGGROUP, 0, 0, 23, 37, [
      (NodeType.KWARGGROUP, 0, 0, 23, 37, [
        (NodeType.KEYWORD, 0, 0, 23, 30, []),
        (NodeType.ARGGROUP, 0, 0, 31, 37, [
          (NodeType.PARGGROUP, 0, 0, 31, 37, [
            (NodeType.ARGUMENT, 0, 0, 31, 37, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 0, 37, 38, []),
  ]),
]),
      ])

  def test_collapse_additional_newlines(self):
    self.do_layout_test("""\
      # The following multiple newlines should be collapsed into a single newline




      cmake_minimum_required(VERSION 2.8.11)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 75, [
  (NodeType.COMMENT, 0, 0, 0, 75, []),
  (NodeType.WHITESPACE, 0, 1, 0, 0, []),
  (NodeType.STATEMENT, 0, 2, 0, 38, [
    (NodeType.FUNNAME, 0, 2, 0, 22, []),
    (NodeType.LPAREN, 0, 2, 22, 23, []),
    (NodeType.ARGGROUP, 0, 2, 23, 37, [
      (NodeType.KWARGGROUP, 0, 2, 23, 37, [
        (NodeType.KEYWORD, 0, 2, 23, 30, []),
        (NodeType.ARGGROUP, 0, 2, 31, 37, [
          (NodeType.PARGGROUP, 0, 2, 31, 37, [
            (NodeType.ARGUMENT, 0, 2, 31, 37, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 2, 37, 38, []),
  ]),
]),
      ])

  def test_multiline_reflow(self):
    self.do_layout_test("""\
      # This multiline-comment should be reflowed
      # into a single comment
      # on one line
      """, [
          (NodeType.BODY, 0, 0, 77, [
              (NodeType.COMMENT, 0, 0, 77, []),
          ])
      ])

  def test_long_args_command_split(self):
    self.do_layout_test("""\
      # This very long command should be split to multiple lines
      set(HEADERS very_long_header_name_a.h very_long_header_name_b.h very_long_header_name_c.h)
      """, [
# pylint: disable=bad-continuation
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 63, [
  (NodeType.COMMENT, 0, 0, 0, 58, []),
  (NodeType.STATEMENT, 0, 1, 0, 63, [
    (NodeType.FUNNAME, 0, 1, 0, 3, []),
    (NodeType.LPAREN, 0, 1, 3, 4, []),
    (NodeType.ARGGROUP, 0, 1, 4, 63, [
      (NodeType.PARGGROUP, 0, 1, 4, 11, [
        (NodeType.ARGUMENT, 0, 1, 4, 11, []),
      ]),
      (NodeType.PARGGROUP, 0, 1, 12, 63, [
        (NodeType.ARGUMENT, 0, 1, 12, 37, []),
        (NodeType.ARGUMENT, 0, 1, 38, 63, []),
        (NodeType.ARGUMENT, 0, 2, 12, 37, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 2, 37, 38, []),
  ]),
]),
      ])

  def test_long_arg_on_newline(self):
    self.do_layout_test("""\
      # This command has a very long argument and can't be aligned with the command
      # end, so it should be moved to a new line with block indent + 1.
      some_long_command_name("Some very long argument that really needs to be on the next line.")
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 77, [
  (NodeType.COMMENT, 0, 0, 0, 77, []),
  (NodeType.STATEMENT, 1, 2, 0, 70, [
    (NodeType.FUNNAME, 0, 2, 0, 22, []),
    (NodeType.LPAREN, 0, 2, 22, 23, []),
    (NodeType.ARGGROUP, 0, 3, 2, 69, [
      (NodeType.PARGGROUP, 0, 3, 2, 69, [
        (NodeType.ARGUMENT, 0, 3, 2, 69, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 3, 69, 70, []),
  ]),
]),
      ])

  def test_argcomment_preserved_and_reflowed(self):
    self.do_layout_test("""\
      set(HEADERS header_a.h header_b.h # This comment should
                                        # be preserved, moreover it should be split
                                        # across two lines.
          header_c.h header_d.h)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 80, [
  (NodeType.STATEMENT, 4, 0, 0, 80, [
    (NodeType.FUNNAME, 0, 0, 0, 3, []),
    (NodeType.LPAREN, 0, 0, 3, 4, []),
    (NodeType.ARGGROUP, 4, 0, 4, 80, [
      (NodeType.PARGGROUP, 0, 0, 4, 11, [
        (NodeType.ARGUMENT, 0, 0, 4, 11, []),
      ]),
      (NodeType.PARGGROUP, 0, 1, 4, 80, [
        (NodeType.ARGUMENT, 0, 1, 4, 14, []),
        (NodeType.ARGUMENT, 0, 1, 15, 80, [
          (NodeType.COMMENT, 0, 1, 26, 80, []),
        ]),
        (NodeType.ARGUMENT, 0, 3, 4, 14, []),
        (NodeType.ARGUMENT, 0, 3, 15, 25, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 3, 25, 26, []),
  ]),
]),
      ])

  def test_complex_nested_stuff(self):
    self.config.format.autosort = False
    self.do_layout_test("""\
      if(foo)
      if(sbar)
      # This comment is in-scope.
      add_library(foo_bar_baz foo.cc bar.cc # this is a comment for arg2
                    # this is more comment for arg2, it should be joined with the first.
          baz.cc) # This comment is part of add_library

      other_command(some_long_argument some_long_argument) # this comment is very long and gets split across some lines

      other_command(some_long_argument some_long_argument some_long_argument) # this comment is even longer and wouldn't make sense to pack at the end of the command so it gets it's own lines
      endif()
      endif()
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 80, [
  (NodeType.FLOW_CONTROL, 0, 0, 0, 80, [
    (NodeType.STATEMENT, 0, 0, 0, 7, [
      (NodeType.FUNNAME, 0, 0, 0, 2, []),
      (NodeType.LPAREN, 0, 0, 2, 3, []),
      (NodeType.ARGGROUP, 0, 0, 3, 6, [
        (NodeType.PARGGROUP, 0, 0, 3, 6, [
          (NodeType.ARGUMENT, 0, 0, 3, 6, []),
        ]),
      ]),
      (NodeType.RPAREN, 0, 0, 6, 7, []),
    ]),
    (NodeType.BODY, 0, 1, 2, 80, [
      (NodeType.FLOW_CONTROL, 0, 1, 2, 80, [
        (NodeType.STATEMENT, 0, 1, 2, 10, [
          (NodeType.FUNNAME, 0, 1, 2, 4, []),
          (NodeType.LPAREN, 0, 1, 4, 5, []),
          (NodeType.ARGGROUP, 0, 1, 5, 9, [
            (NodeType.PARGGROUP, 0, 1, 5, 9, [
              (NodeType.ARGUMENT, 0, 1, 5, 9, []),
            ]),
          ]),
          (NodeType.RPAREN, 0, 1, 9, 10, []),
        ]),
        (NodeType.BODY, 0, 2, 4, 80, [
          (NodeType.COMMENT, 0, 2, 4, 31, []),
          (NodeType.STATEMENT, 4, 3, 4, 74, [
            (NodeType.FUNNAME, 0, 3, 4, 15, []),
            (NodeType.LPAREN, 0, 3, 15, 16, []),
            (NodeType.ARGGROUP, 4, 4, 6, 74, [
              (NodeType.PARGGROUP, 0, 4, 6, 17, [
                (NodeType.ARGUMENT, 0, 4, 6, 17, []),
              ]),
              (NodeType.PARGGROUP, 0, 5, 6, 74, [
                (NodeType.ARGUMENT, 0, 5, 6, 12, []),
                (NodeType.ARGUMENT, 0, 5, 13, 48, [
                  (NodeType.COMMENT, 0, 5, 20, 48, []),
                ]),
                (NodeType.COMMENT, 0, 6, 6, 74, []),
                (NodeType.ARGUMENT, 0, 7, 6, 12, []),
              ]),
            ]),
            (NodeType.RPAREN, 0, 7, 12, 13, []),
            (NodeType.COMMENT, 0, 7, 14, 51, []),
          ]),
          (NodeType.WHITESPACE, 0, 8, 4, 0, []),
          (NodeType.STATEMENT, 1, 9, 4, 76, [
            (NodeType.FUNNAME, 0, 9, 4, 17, []),
            (NodeType.LPAREN, 0, 9, 17, 18, []),
            (NodeType.ARGGROUP, 0, 10, 6, 43, [
              (NodeType.PARGGROUP, 0, 10, 6, 43, [
                (NodeType.ARGUMENT, 0, 10, 6, 24, []),
                (NodeType.ARGUMENT, 0, 10, 25, 43, []),
              ]),
            ]),
            (NodeType.RPAREN, 0, 10, 43, 44, []),
            (NodeType.COMMENT, 0, 10, 45, 76, []),
          ]),
          (NodeType.WHITESPACE, 0, 12, 4, 0, []),
          (NodeType.STATEMENT, 1, 13, 4, 80, [
            (NodeType.FUNNAME, 0, 13, 4, 17, []),
            (NodeType.LPAREN, 0, 13, 17, 18, []),
            (NodeType.ARGGROUP, 0, 14, 6, 62, [
              (NodeType.PARGGROUP, 0, 14, 6, 62, [
                (NodeType.ARGUMENT, 0, 14, 6, 24, []),
                (NodeType.ARGUMENT, 0, 14, 25, 43, []),
                (NodeType.ARGUMENT, 0, 14, 44, 62, []),
              ]),
            ]),
            (NodeType.RPAREN, 0, 14, 62, 63, []),
            (NodeType.COMMENT, 0, 14, 64, 80, []),
          ]),
        ]),
        (NodeType.STATEMENT, 0, 23, 2, 9, [
          (NodeType.FUNNAME, 0, 23, 2, 7, []),
          (NodeType.LPAREN, 0, 23, 7, 8, []),
          (NodeType.ARGGROUP, 0, 23, 8, 8, []),
          (NodeType.RPAREN, 0, 23, 8, 9, []),
        ]),
      ]),
    ]),
    (NodeType.STATEMENT, 0, 24, 0, 7, [
      (NodeType.FUNNAME, 0, 24, 0, 5, []),
      (NodeType.LPAREN, 0, 24, 5, 6, []),
      (NodeType.ARGGROUP, 0, 24, 6, 6, []),
      (NodeType.RPAREN, 0, 24, 6, 7, []),
    ]),
  ]),
]),
      ])

  def test_custom_command(self):
    self.do_layout_test("""\
      # This very long command should be broken up along keyword arguments
      foo(nonkwarg_a nonkwarg_b HEADERS a.h b.h c.h d.h e.h f.h SOURCES a.cc b.cc d.cc DEPENDS foo bar baz)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 68, [
  (NodeType.COMMENT, 0, 0, 0, 68, []),
  (NodeType.STATEMENT, 4, 1, 0, 35, [
    (NodeType.FUNNAME, 0, 1, 0, 3, []),
    (NodeType.LPAREN, 0, 1, 3, 4, []),
    (NodeType.ARGGROUP, 4, 1, 4, 35, [
      (NodeType.PARGGROUP, 0, 1, 4, 25, [
        (NodeType.ARGUMENT, 0, 1, 4, 14, []),
        (NodeType.ARGUMENT, 0, 1, 15, 25, []),
      ]),
      (NodeType.KWARGGROUP, 0, 2, 4, 35, [
        (NodeType.KEYWORD, 0, 2, 4, 11, []),
        (NodeType.ARGGROUP, 0, 2, 12, 35, [
          (NodeType.PARGGROUP, 0, 2, 12, 35, [
            (NodeType.ARGUMENT, 0, 2, 12, 15, []),
            (NodeType.ARGUMENT, 0, 2, 16, 19, []),
            (NodeType.ARGUMENT, 0, 2, 20, 23, []),
            (NodeType.ARGUMENT, 0, 2, 24, 27, []),
            (NodeType.ARGUMENT, 0, 2, 28, 31, []),
            (NodeType.ARGUMENT, 0, 2, 32, 35, []),
          ]),
        ]),
      ]),
      (NodeType.KWARGGROUP, 0, 3, 4, 26, [
        (NodeType.KEYWORD, 0, 3, 4, 11, []),
        (NodeType.ARGGROUP, 0, 3, 12, 26, [
          (NodeType.PARGGROUP, 0, 3, 12, 26, [
            (NodeType.ARGUMENT, 0, 3, 12, 16, []),
            (NodeType.ARGUMENT, 0, 3, 17, 21, []),
            (NodeType.ARGUMENT, 0, 3, 22, 26, []),
          ]),
        ]),
      ]),
      (NodeType.KWARGGROUP, 0, 4, 4, 15, [
        (NodeType.KEYWORD, 0, 4, 4, 11, []),
        (NodeType.ARGGROUP, 0, 4, 12, 15, [
          (NodeType.PARGGROUP, 0, 4, 12, 15, [
            (NodeType.ARGUMENT, 0, 4, 12, 15, []),
          ]),
        ]),
      ]),
      (NodeType.PARGGROUP, 0, 5, 4, 11, [
        (NodeType.FLAG, 0, 5, 4, 7, []),
        (NodeType.FLAG, 0, 5, 8, 11, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 5, 11, 12, []),
  ]),
]),
      ])

  def test_multiline_string(self):
    self.do_layout_test("""\
      foo(some_arg some_arg "
          This string is on multiple lines
      ")
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 36, [
  (NodeType.STATEMENT, 0, 0, 0, 36, [
    (NodeType.FUNNAME, 0, 0, 0, 3, []),
    (NodeType.LPAREN, 0, 0, 3, 4, []),
    (NodeType.ARGGROUP, 0, 0, 4, 36, [
      (NodeType.PARGGROUP, 0, 0, 4, 36, [
        (NodeType.ARGUMENT, 0, 0, 4, 12, []),
        (NodeType.ARGUMENT, 0, 0, 13, 21, []),
        (NodeType.ARGUMENT, 0, 0, 22, 36, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 2, 1, 2, []),
  ]),
]),
      ])

  def test_nested_parens(self):
    self.do_layout_test("""\
      if((NOT HELLO) OR (NOT EXISTS ${WORLD}))
        message(WARNING "something is wrong")
        set(foobar FALSE)
      endif()
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 40, [
  (NodeType.FLOW_CONTROL, 0, 0, 0, 40, [
    (NodeType.STATEMENT, 0, 0, 0, 40, [
      (NodeType.FUNNAME, 0, 0, 0, 2, []),
      (NodeType.LPAREN, 0, 0, 2, 3, []),
      (NodeType.ARGGROUP, 0, 0, 3, 39, [
        (NodeType.PARENGROUP, 0, 0, 3, 14, [
          (NodeType.LPAREN, 0, 0, 3, 4, []),
          (NodeType.ARGGROUP, 0, 0, 4, 13, [
            (NodeType.PARGGROUP, 0, 0, 4, 13, [
              (NodeType.FLAG, 0, 0, 4, 7, []),
              (NodeType.ARGUMENT, 0, 0, 8, 13, []),
            ]),
          ]),
          (NodeType.RPAREN, 0, 0, 13, 14, []),
        ]),
        (NodeType.KWARGGROUP, 0, 0, 15, 39, [
          (NodeType.KEYWORD, 0, 0, 15, 17, []),
          (NodeType.ARGGROUP, 0, 0, 18, 39, [
            (NodeType.PARENGROUP, 0, 0, 18, 39, [
              (NodeType.LPAREN, 0, 0, 18, 19, []),
              (NodeType.ARGGROUP, 0, 0, 19, 38, [
                (NodeType.PARGGROUP, 0, 0, 19, 38, [
                  (NodeType.FLAG, 0, 0, 19, 22, []),
                  (NodeType.FLAG, 0, 0, 23, 29, []),
                  (NodeType.ARGUMENT, 0, 0, 30, 38, []),
                ]),
              ]),
              (NodeType.RPAREN, 0, 0, 38, 39, []),
            ]),
          ]),
        ]),
      ]),
      (NodeType.RPAREN, 0, 0, 39, 40, []),
    ]),
    (NodeType.BODY, 0, 1, 2, 39, [
      (NodeType.STATEMENT, 0, 1, 2, 39, [
        (NodeType.FUNNAME, 0, 1, 2, 9, []),
        (NodeType.LPAREN, 0, 1, 9, 10, []),
        (NodeType.ARGGROUP, 0, 1, 10, 38, [
          (NodeType.KWARGGROUP, 0, 1, 10, 38, [
            (NodeType.KEYWORD, 0, 1, 10, 17, []),
            (NodeType.ARGGROUP, 0, 1, 18, 38, [
              (NodeType.PARGGROUP, 0, 1, 18, 38, [
                (NodeType.ARGUMENT, 0, 1, 18, 38, []),
              ]),
            ]),
          ]),
        ]),
        (NodeType.RPAREN, 0, 1, 38, 39, []),
      ]),
      (NodeType.STATEMENT, 0, 2, 2, 19, [
        (NodeType.FUNNAME, 0, 2, 2, 5, []),
        (NodeType.LPAREN, 0, 2, 5, 6, []),
        (NodeType.ARGGROUP, 0, 2, 6, 18, [
          (NodeType.PARGGROUP, 0, 2, 6, 12, [
            (NodeType.ARGUMENT, 0, 2, 6, 12, []),
          ]),
          (NodeType.PARGGROUP, 0, 2, 13, 18, [
            (NodeType.ARGUMENT, 0, 2, 13, 18, []),
          ]),
        ]),
        (NodeType.RPAREN, 0, 2, 18, 19, []),
      ]),
    ]),
    (NodeType.STATEMENT, 0, 3, 0, 7, [
      (NodeType.FUNNAME, 0, 3, 0, 5, []),
      (NodeType.LPAREN, 0, 3, 5, 6, []),
      (NodeType.ARGGROUP, 0, 3, 6, 6, []),
      (NodeType.RPAREN, 0, 3, 6, 7, []),
    ]),
  ]),
]),
      ])

  def test_comment_after_command(self):
    with self.subTest(sub=1):
      self.do_layout_test("""\
        foo_command() # comment
        """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 23, [
  (NodeType.STATEMENT, 0, 0, 0, 23, [
    (NodeType.FUNNAME, 0, 0, 0, 11, []),
    (NodeType.LPAREN, 0, 0, 11, 12, []),
    (NodeType.ARGGROUP, 0, 0, 12, 12, []),
    (NodeType.RPAREN, 0, 0, 12, 13, []),
    (NodeType.COMMENT, 0, 0, 14, 23, []),
  ]),
]),
        ])

    with self.subTest(sub=2):
      self.do_layout_test("""\
        foo_command() # this is a long comment that exceeds the desired page width and will be wrapped to a newline
        """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 78, [
  (NodeType.STATEMENT, 0, 0, 0, 78, [
    (NodeType.FUNNAME, 0, 0, 0, 11, []),
    (NodeType.LPAREN, 0, 0, 11, 12, []),
    (NodeType.ARGGROUP, 0, 0, 12, 12, []),
    (NodeType.RPAREN, 0, 0, 12, 13, []),
    (NodeType.COMMENT, 0, 0, 14, 78, []),
  ]),
]),

        ])

  def test_arg_just_fits(self):
    """
    Ensure that if an argument *just* fits that it isn't superfluously wrapped
    """

    with self.subTest(chars=81):
      self.do_layout_test("""\
      message(FATAL_ERROR "81 character line ----------------------------------------")
    """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 75, [
  (NodeType.STATEMENT, 1, 0, 0, 75, [
    (NodeType.FUNNAME, 0, 0, 0, 7, []),
    (NodeType.LPAREN, 0, 0, 7, 8, []),
    (NodeType.ARGGROUP, 0, 1, 2, 74, [
      (NodeType.KWARGGROUP, 0, 1, 2, 74, [
        (NodeType.KEYWORD, 0, 1, 2, 13, []),
        (NodeType.ARGGROUP, 0, 1, 14, 74, [
          (NodeType.PARGGROUP, 0, 1, 14, 74, [
            (NodeType.ARGUMENT, 0, 1, 14, 74, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 1, 74, 75, []),
  ]),
]),
    ])

    with self.subTest(chars=100, with_prefix=True):
      self.do_layout_test("""\
      message(FATAL_ERROR
              "100 character line ----------------------------------------------------------"
      ) # Closing parenthesis is indented one space!
    """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 83, [
  (NodeType.STATEMENT, 5, 0, 0, 83, [
    (NodeType.FUNNAME, 0, 0, 0, 7, []),
    (NodeType.LPAREN, 0, 0, 7, 8, []),
    (NodeType.ARGGROUP, 5, 1, 2, 83, [
      (NodeType.KWARGGROUP, 5, 1, 2, 83, [
        (NodeType.KEYWORD, 0, 1, 2, 13, []),
        (NodeType.ARGGROUP, 5, 2, 4, 83, [
          (NodeType.PARGGROUP, 4, 2, 4, 83, [
            (NodeType.ARGUMENT, 0, 2, 4, 83, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 3, 0, 1, []),
    (NodeType.COMMENT, 0, 3, 2, 46, []),
  ]),
]),
    ])

    with self.subTest(chars=100):
      self.do_layout_test("""\
      message(
        "100 character line ----------------------------------------------------------------------"
        ) # Closing parenthesis is indented one space!
    """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 93, [
  (NodeType.STATEMENT, 5, 0, 0, 93, [
    (NodeType.FUNNAME, 0, 0, 0, 7, []),
    (NodeType.LPAREN, 0, 0, 7, 8, []),
    (NodeType.ARGGROUP, 5, 1, 2, 93, [
      (NodeType.PARGGROUP, 4, 1, 2, 93, [
        (NodeType.ARGUMENT, 0, 1, 2, 93, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 2, 0, 1, []),
    (NodeType.COMMENT, 0, 2, 2, 46, []),
  ]),
]),
    ])

  def test_string_preserved_during_split(self):
    self.do_layout_test("""\
      # The string in this command should not be split
      set_target_properties(foo bar baz PROPERTIES COMPILE_FLAGS "-std=c++11 -Wall -Wextra")
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 72, [
  (NodeType.COMMENT, 0, 0, 0, 48, []),
  (NodeType.STATEMENT, 0, 1, 0, 72, [
    (NodeType.FUNNAME, 0, 1, 0, 21, []),
    (NodeType.LPAREN, 0, 1, 21, 22, []),
    (NodeType.ARGGROUP, 0, 1, 22, 71, [
      (NodeType.PARGGROUP, 0, 1, 22, 33, [
        (NodeType.ARGUMENT, 0, 1, 22, 25, []),
        (NodeType.ARGUMENT, 0, 1, 26, 29, []),
        (NodeType.ARGUMENT, 0, 1, 30, 33, []),
      ]),
      (NodeType.KWARGGROUP, 0, 1, 34, 71, [
        (NodeType.KEYWORD, 0, 1, 34, 44, []),
        (NodeType.ARGGROUP, 0, 1, 45, 71, [
          (NodeType.PARGGROUP, 0, 1, 45, 71, [
            (NodeType.ARGUMENT, 0, 1, 45, 58, []),
            (NodeType.ARGUMENT, 0, 2, 45, 71, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 2, 71, 72, []),
  ]),
]),
      ])

  def test_while(self):
    self.do_layout_test("""\
      while(forbarbaz arg1 arg2, arg3)
        message(hello ${foobarbaz})
      endwhile()
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 32, [
  (NodeType.FLOW_CONTROL, 0, 0, 0, 32, [
    (NodeType.STATEMENT, 0, 0, 0, 32, [
      (NodeType.FUNNAME, 0, 0, 0, 5, []),
      (NodeType.LPAREN, 0, 0, 5, 6, []),
      (NodeType.ARGGROUP, 0, 0, 6, 31, [
        (NodeType.PARGGROUP, 0, 0, 6, 31, [
          (NodeType.ARGUMENT, 0, 0, 6, 15, []),
          (NodeType.ARGUMENT, 0, 0, 16, 20, []),
          (NodeType.ARGUMENT, 0, 0, 21, 26, []),
          (NodeType.ARGUMENT, 0, 0, 27, 31, []),
        ]),
      ]),
      (NodeType.RPAREN, 0, 0, 31, 32, []),
    ]),
    (NodeType.BODY, 0, 1, 2, 29, [
      (NodeType.STATEMENT, 0, 1, 2, 29, [
        (NodeType.FUNNAME, 0, 1, 2, 9, []),
        (NodeType.LPAREN, 0, 1, 9, 10, []),
        (NodeType.ARGGROUP, 0, 1, 10, 28, [
          (NodeType.PARGGROUP, 0, 1, 10, 28, [
            (NodeType.ARGUMENT, 0, 1, 10, 15, []),
            (NodeType.ARGUMENT, 0, 1, 16, 28, []),
          ]),
        ]),
        (NodeType.RPAREN, 0, 1, 28, 29, []),
      ]),
    ]),
    (NodeType.STATEMENT, 0, 2, 0, 10, [
      (NodeType.FUNNAME, 0, 2, 0, 8, []),
      (NodeType.LPAREN, 0, 2, 8, 9, []),
      (NodeType.ARGGROUP, 0, 2, 9, 9, []),
      (NodeType.RPAREN, 0, 2, 9, 10, []),
    ]),
  ]),
]),
      ])

  def test_keyword_comment(self):
    self.do_layout_test("""\
      find_package(package REQUIRED
                   COMPONENTS # --------------------------------------
                              # @TODO: This has to be filled manually
                              # --------------------------------------
                              this_is_a_really_long_word_foo)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 53, [
  (NodeType.STATEMENT, 4, 0, 0, 53, [
    (NodeType.FUNNAME, 0, 0, 0, 12, []),
    (NodeType.LPAREN, 0, 0, 12, 13, []),
    (NodeType.ARGGROUP, 4, 1, 2, 53, [
      (NodeType.PARGGROUP, 0, 1, 2, 18, [
        (NodeType.ARGUMENT, 0, 1, 2, 9, []),
        (NodeType.FLAG, 0, 1, 10, 18, []),
      ]),
      (NodeType.KWARGGROUP, 4, 2, 2, 53, [
        (NodeType.KEYWORD, 0, 2, 2, 12, []),
        (NodeType.ARGGROUP, 4, 2, 13, 53, [
          (NodeType.COMMENT, 0, 2, 13, 53, []),
          (NodeType.PARGGROUP, 0, 5, 13, 43, [
            (NodeType.ARGUMENT, 0, 5, 13, 43, []),
          ]),
        ]),
      ]),
    ]),
    (NodeType.RPAREN, 0, 5, 43, 44, []),
  ]),
]),
      ])

  def test_sortable_set(self):
    self.do_layout_test("""\
      set(SOURCES #[[cmf:sortable]] foo.cc bar.cc baz.cc)
      """, [
# pylint: disable=bad-continuation
# noqa: E122
(NodeType.BODY, 0, 0, 0, 51, [
  (NodeType.STATEMENT, 0, 0, 0, 51, [
    (NodeType.FUNNAME, 0, 0, 0, 3, []),
    (NodeType.LPAREN, 0, 0, 3, 4, []),
    (NodeType.ARGGROUP, 0, 0, 4, 50, [
      (NodeType.PARGGROUP, 0, 0, 4, 11, [
        (NodeType.ARGUMENT, 0, 0, 4, 11, []),
      ]),
      (NodeType.PARGGROUP, 0, 0, 12, 50, [
        (NodeType.COMMENT, 0, 0, 12, 29, []),
        (NodeType.ARGUMENT, 0, 0, 30, 36, []),
        (NodeType.ARGUMENT, 0, 0, 37, 43, []),
        (NodeType.ARGUMENT, 0, 0, 44, 50, []),
      ]),
    ]),
    (NodeType.RPAREN, 0, 0, 50, 51, []),
  ]),
]),
      ])


if __name__ == '__main__':
  format_str = '[%(levelname)-4s] %(filename)s:%(lineno)-3s: %(message)s'
  logging.basicConfig(level=logging.DEBUG,
                      format=format_str,
                      datefmt='%Y-%m-%d %H:%M:%S',
                      filemode='w')
  unittest.main()



"""
Generate stubs for missing parsers
"""

import argparse
import io
import os
import subprocess
import sys

from cmakelang import parse

TEMPLATE = '''

def parse_{0}(ctx, tokens, breakstack):
  """
  ::

    {0}

  :see: https://cmake.org/cmake/help/latest/command/{0}.html
  """
'''


def main():
  parser = argparse.ArgumentParser(description=__doc__)
  parser.add_argument("outfile", nargs="?", default="-")
  args = parser.parse_args()

  missing_commands = []
  proc = subprocess.Popen(
      ["cmake", "--help-command-list"],
      stdout=subprocess.PIPE)

  parse_db = parse.funs.get_parse_db()

  with proc.stdout as infile:
    for line in infile:
      command = line.strip().decode("utf-8")
      if command not in parse_db and not command.startswith("end"):
        missing_commands.append(command)
  proc.wait()

  outfile_path = args.outfile
  if outfile_path == "-":
    outfile_path = os.dup(sys.stdout.fileno())

  with io.open(outfile_path, "w", encoding="utf-8") as outfile:
    for command in missing_commands:
      outfile.write(TEMPLATE.format(command))

    outfile.write("\n\ndef populate_db(parse_db):\n")
    for command in missing_commands:
      outfile.write('  parse_db["{0}"] = parse_{0}\n'.format(command))


if __name__ == "__main__":
  main()



# -*- coding: utf-8 -*-
"""
Parse cmake listfiles, find function and macro declarations, and generate
parsers for them.
"""
from __future__ import print_function, unicode_literals

import argparse
import collections
import io
import json
import logging
import os
import pprint
import re
import sys

import cmakelang
from cmakelang.format import __main__
from cmakelang import configuration
from cmakelang import lex
from cmakelang import parse
from cmakelang.parse.body_nodes import BodyNode
from cmakelang.parse.common import TreeNode
from cmakelang.parse.statement_node import StatementNode
from cmakelang.parse.funs.set import SetFnNode

logger = logging.getLogger(__name__)


def find_statements_in_subtree(subtree, funnames):
  """
  Return a generator that yields all statements in the `subtree` which match
  the provided set of `funnames`.
  """
  if isinstance(subtree, (list, tuple)):
    queue = subtree
  else:
    queue = [subtree]

  while queue:
    node = queue.pop(0)

    if isinstance(node, SetFnNode) and "set" in funnames:
      yield node
      continue

    if isinstance(node, StatementNode):
      if node.get_funname() in funnames:
        yield node
        continue

    for child in node.children:
      if isinstance(child, TreeNode):
        queue.append(child)


class VarSub(object):
  def __init__(self, variables):
    self._vars = variables

  def __call__(self, match):
    return self._vars.get(match.group(1), "")


def replace_varrefs(value, variables):
  """
  Replace cmake variable dereferences in the string, given a dictionary of
  currently assigned variables.
  """
  regex = re.compile(r"\$\{([\w_]+)\}")
  repl = VarSub(variables)
  while regex.search(value):
    value = regex.sub(repl, value)
  return value


def process_set_statement(argtree, variables):
  """
  Process a set() statement, updating the variable assignments accordingly
  """
  varname = replace_varrefs(argtree.varname.spelling, variables)
  if not argtree.value_group:
    variables.pop(varname, None)
    return

  setargs = argtree.value_group.get_tokens(kind="semantic")
  valuestr = ";".join(arg.spelling.strip('"') for arg in setargs)
  variables[varname] = replace_varrefs(valuestr, variables)


def process_defn_body(body, out):
  assert isinstance(body, BodyNode)
  variables = {}
  for child in find_statements_in_subtree(
      body, ["list", "cmake_parse_arguments", "set"]):

    if isinstance(child, StatementNode):
      if child.get_funname() == "set":
        process_set_statement(child.argtree, variables)
        continue

      if child.get_funname() == "list":
        # TODO(josh): implement list processor
        continue

      if child.get_funname() == "cmake_parse_arguments":
        pargs = child.argtree.get_tokens(kind="semantic")
        pargs.pop(0)
        kwvargs = replace_varrefs(
            pargs.pop(0).spelling.strip('"'), variables).split(";")
        onevargs = replace_varrefs(
            pargs.pop(0).spelling.strip('"'), variables).split(";")
        multivargs = replace_varrefs(
            pargs.pop(0).spelling.strip('"'), variables).split(";")

        pargs = out["pargs"]
        nargs = pargs["nargs"]
        if nargs == 0:
          pargs["nargs"] = "*"
        else:
          pargs["nargs"] = "{}+".format(nargs)

        pargs["flags"] = flags = []
        for flag in kwvargs:
          flag = flag.strip().upper()
          if flag:
            flags.append(flag)

        out["kwargs"] = kwargs = {}

        for kwarg in onevargs:
          kwarg = kwarg.strip().upper()
          if kwarg:
            kwargs[kwarg] = 1
        for kwarg in multivargs:
          kwarg = kwarg.strip().upper()
          if kwarg:
            kwargs[kwarg] = "+"
        return
  return


def process_defn(statement):
  """
  Process one function or macro definition
  """
  semantic_tokens = statement.argtree.parg_groups[0].get_tokens(kind="semantic")

  canonical_spelling = semantic_tokens[0].spelling
  fnname = canonical_spelling.lower()
  argnames = [token.spelling.lower() for token in semantic_tokens[1:]]

  out = {
      "pargs": {
          "nargs": len(argnames)
      }
  }

  if (canonical_spelling.lower() != canonical_spelling and
      canonical_spelling.upper() != canonical_spelling):
    # mixed case
    out["spelling"] = canonical_spelling

  process_defn_body(statement.parent.children[1], out)
  return fnname, out


def process_tree(parse_tree):
  """
  Process the resulting parse tree
  """
  out = collections.OrderedDict()
  for statement in find_statements_in_subtree(
      parse_tree, ("function", "macro")):
    fnname, spec = process_defn(statement)
    if fnname.startswith("_"):
      continue
    out[fnname] = spec
  return out


def process_file(config, infile_content):
  """
  Parse the input cmake file, return the parse tree
  """

  if config.format.line_ending == 'auto':
    detected = __main__.detect_line_endings(infile_content)
    config = config.clone()
    config.set_line_ending(detected)

  tokens = lex.tokenize(infile_content)
  parse_db = parse.funs.get_parse_db()
  parse_db.update(parse.funs.get_funtree(config.parse.fn_spec))
  ctx = parse.ParseContext(parse_db, config=config)
  parse_tree = parse.parse(tokens, ctx)
  parse_tree.build_ancestry()
  return parse_tree


def setup_argparse(argparser):
  argparser.add_argument('-v', '--version', action='version',
                         version=cmakelang.__version__)
  argparser.add_argument(
      '-l', '--log-level', default="info",
      choices=["error", "warning", "info", "debug"])
  argparser.add_argument(
      '-o', '--outfile-path', default=None,
      help='Write results to this file. Default is stdout.')
  argparser.add_argument(
      '-f', '--output-format', default="python",
      choices=["json", "yaml", "python"])
  argparser.add_argument('infilepaths', nargs='*')


USAGE_STRING = """
cmake-genparsers [-h] [-o OUTFILE_PATH] infilepath [infilepath ...]
"""


def main():
  """Parse arguments, open files, start work."""
  logging.basicConfig(level=logging.INFO, format="%(levelname)s %(message)s")

  argparser = argparse.ArgumentParser(
      description=__doc__,
      formatter_class=argparse.RawDescriptionHelpFormatter,
      usage=USAGE_STRING)

  setup_argparse(argparser)
  args = argparser.parse_args()
  logging.getLogger().setLevel(getattr(logging, args.log_level.upper()))

  if '-' in args.infilepaths:
    assert len(args.infilepaths) == 1, \
        "You cannot mix stdin as an input with other input files"

  if args.outfile_path is None:
    args.outfile_path = '-'

  outfile_arg = args.outfile_path
  if outfile_arg == '-':
    outfile_arg = os.dup(sys.stdout.fileno())

  returncode = 0
  outdict = collections.OrderedDict()
  for infile_path in args.infilepaths:
    print(infile_path)
    cfg = configuration.Configuration()
    if infile_path == '-':
      infile_path = os.dup(sys.stdin.fileno())

    try:
      infile = io.open(
          infile_path, mode='r', encoding=cfg.encode.input_encoding, newline='')
    except (IOError, OSError):
      logger.error("Failed to open %s for read", infile_path)
      returncode = 1

    try:
      with infile:
        intext = infile.read()
    except UnicodeDecodeError:
      logger.error(
          "Unable to read %s as %s", infile_path, cfg.encode.input_encoding)

    try:
      parse_tree = process_file(cfg, intext)
      cmd_spec = process_tree(parse_tree)
      outdict.update(cmd_spec)
    except:
      logger.warning('While processing %s', infile_path)
      raise

  outfile = io.open(outfile_arg, mode='w', encoding="utf-8", newline='')
  if args.output_format == "json":
    json.dump(outdict, outfile, indent=2)
  elif args.output_format == "yaml":
    import yaml
    __main__.yaml_register_odict(yaml.SafeDumper)
    __main__.yaml_register_odict(yaml.Dumper)
    yaml.dump(outdict,
              outfile, indent=2,
              default_flow_style=False, sort_keys=False)
  elif args.output_format == "python":
    ppr = pprint.PrettyPrinter(indent=2, width=80)
    outfile.write(ppr.pformat(dict(outdict)))
  else:
    logger.error("Unrecognized output format {}".format(args.output_format))
  outfile.write("\n")
  outfile.close()
  return returncode


if __name__ == "__main__":
  sys.exit(main())



# pylint: disable=bad-continuation
from __future__ import unicode_literals
import unittest

from cmakelang.command_tests import TestBase
from cmakelang.parse.common import NodeType


class TestAddLibraryCommand(TestBase):
  """
  Test various examples of add_library()
  """
  kExpectNumSidecarTests = 9

  def test_parse_with_concluding_comments(self):
    self.source_str = """\
add_library(foo STATIC EXCLUDE_FROM_ALL foo.cc bar.cc
            # This is a concluding comment
            )
"""
    self.expect_parse = [
      (NodeType.BODY, [
        (NodeType.STATEMENT, [
          (NodeType.FUNNAME, []),
          (NodeType.LPAREN, []),
          (NodeType.ARGGROUP, [
            (NodeType.PARGGROUP, [
              (NodeType.ARGUMENT, []),
              (NodeType.FLAG, []),
              (NodeType.FLAG, []),
            ]),
            (NodeType.PARGGROUP, [
              (NodeType.ARGUMENT, []),
              (NodeType.ARGUMENT, []),
            ]),
            (NodeType.COMMENT, []),
          ]),
          (NodeType.RPAREN, []),
        ]),
        (NodeType.WHITESPACE, []),
      ]),
    ]

  def test_sort_arguments(self):
    self.config.format.autosort = True
    self.source_str = """\
add_library(foobar STATIC sourcefile_04.cc
  sourcefile_03.cc sourcefile_01.cc sourcefile_02.cc)
"""

    self.expect_format = """\
add_library(foobar STATIC sourcefile_01.cc sourcefile_02.cc sourcefile_03.cc
                          sourcefile_04.cc)
"""


if __name__ == '__main__':
  unittest.main()



import os
import subprocess
import sys
import tempfile
import unittest

ROOTDIR = os.sep.join(os.path.realpath(__file__).split(os.sep)[:-4])


def iter_testfiles():
  testsdir = os.path.join(ROOTDIR, "cmakelang", "command_tests")
  for filename in os.listdir(testsdir):
    if filename.endswith("_tests.cmake"):
      yield os.path.join("cmakelang", "command_tests", filename)


def make_test_fun(testname, filepath):
  def test_fun(testcase):
    with tempfile.NamedTemporaryFile(mode="wb") as outfile:
      result = subprocess.call(
          [sys.executable, "-Bm", "cmakelang.lint", filepath],
          cwd=ROOTDIR, stdout=outfile)
    testcase.assertNotEqual(2, result)

  if sys.version_info < (3, 0, 0):
    # In python 2.7 test_name is a unicode object. We need to convert it to
    # a string.
    testname = testname.encode("utf-8")
  test_fun.__name__ = testname
  test_fun.__doc__ = " ".join(testname.split("_")[1:])
  return test_fun


def gen_test_class():
  defn = {}
  for filepath in iter_testfiles():
    basename = filepath.split(os.sep)[-1][:-len("_tests.cmake")]
    testname = "test_" + basename
    defn[testname] = make_test_fun(testname, filepath)
  return type("TestFormatFiles", (unittest.TestCase,), defn)


TestFormatFiles = gen_test_class()

if __name__ == "__main__":
  sys.exit(unittest.main())



from __future__ import print_function
from __future__ import unicode_literals

import enum
import logging
import re

logger = logging.getLogger(__name__)


class TokenType(enum.IntEnum):

  LSQ_BRACKET = 0
  RSQ_BRACKET = 1
  LA_BRACKET = 2
  RA_BRACKET = 3
  PIPE = 4
  BIGNAME = 5
  SMALLNAME = 6
  ELLIPSIS = 7
  WHITESPACE = 8
  LPAREN = 9
  RPAREN = 10


class SourceLocation(tuple):
  """
  Named tuple of (line, col, offset)
  """

  # def __init__(self, line, col, offset):
  #   super(SourceLocation, self).__init__((line, col, offset))

  @property
  def line(self):
    return self[0]

  @property
  def col(self):
    return self[1]

  @property
  def offset(self):
    return self[2]

  def __repr__(self):
    return '{}:{}'.format(self.line, self.col)


class Token(object):
  """
  Lexical unit of a listfile.
  """

  def __init__(self, tok_type, spelling, index, begin, end):
    self.type = tok_type
    self.spelling = spelling
    self.index = index
    self.begin = begin
    self.end = end

  @property
  def content(self):
    return self.spelling

  # TODO(josh): get rid of this? Is it used or did I accidentally add it when
  # I meant to add get_location()? Or should this be a property?
  def location(self):
    return self.begin

  def get_location(self):
    return self.begin

  def count_newlines(self):
    return self.spelling.count('\n')

  def __repr__(self):
    """A string representation of this token."""
    return ("Token({0}{2}:{3} {1})").format(
        repr(self.spelling), self.type.name, *self.begin)


def tokenize_inner(contents):
  """
  Scan a string and return a list of Token objects representing the contents
  of the cmake listfile.
  """

  # Regexes are in priority order. Changing the order may alter the
  # behavior of the lexer
  scanner = re.Scanner([
      # Optional group brackets
      (r"\[", lambda s, t: (TokenType.LSQ_BRACKET, t)),
      (r"\]", lambda s, t: (TokenType.RSQ_BRACKET, t)),
      # Mandatory group brackets
      ("<", lambda s, t: (TokenType.LA_BRACKET, t)),
      (">", lambda s, t: (TokenType.RA_BRACKET, t)),
      # Parenthesis
      (r"\(", lambda s, t: (TokenType.LPAREN, t)),
      (r"\)", lambda s, t: (TokenType.RPAREN, t)),
      # Pipe character
      (r"\|", lambda s, t: (TokenType.PIPE, t)),
      # uppercase name
      (r"[A-Z0-9_]+", lambda s, t: (TokenType.BIGNAME, t)),
      # lowercase name
      (r"[a-z0-9_\-]+", lambda s, t: (TokenType.SMALLNAME, t)),
      # ellipsis
      (r"\.\.\.", lambda s, t: (TokenType.ELLIPSIS, t)),
      # whitespace
      (r"\s+", lambda s, t: (TokenType.WHITESPACE, t)),
  ], re.DOTALL)

  tokens, remainder = scanner.scan(contents)
  if remainder:
    raise ValueError("Unparsed tokens: {}".format(remainder))
  return tokens


def annotate_tokens(tokens):
  """
  Add source location and index information to tokens
  """

  # Now add line, column, and serial number to token objects. We get lineno
  # by maintaining a running count of newline characters encountered among
  # tokens so far, and column count by splitting the most recent token on
  # it's right most newline. Note that line and numbers are 1-indexed to match
  # up with editors but column numbers are zero indexed because its fun to be
  # inconsistent.
  lineno = 1
  col = 0
  offset = 0
  tokens_return = []
  for tok_index, (tok_type, spelling) in enumerate(tokens):
    begin = SourceLocation((lineno, col, offset))
    newlines = spelling.count('\n')
    lineno += newlines
    if newlines:
      col = len(spelling.rsplit('\n', 1)[1])
    else:
      col += len(spelling)

    offset += len(bytearray(spelling, 'utf-8'))
    tokens_return.append(Token(
        tok_type=tok_type,
        spelling=spelling,
        index=tok_index,
        begin=begin,
        end=SourceLocation((lineno, col, offset))
    ))
  return tokens_return


def tokenize(content):
  if "(" not in content or ")" not in content:
    raise ValueError("Missing parenthesis")

  tokens = tokenize_inner(content)
  return annotate_tokens(tokens)


def main():
  """
  Dump tokenized usage text for debugging
  """
  import argparse
  import io
  import os
  import sys

  parser = argparse.ArgumentParser(description=__doc__)
  parser.add_argument('infile')
  args = parser.parse_args()

  infile_arg = args.infile
  if args.infile == "-":
    infile_arg = os.dup(sys.stdin.fileno())

  with io.open(infile_arg, 'r') as infile:
    tokens = tokenize(infile.read())
  print('\n'.join(str(x) for x in tokens))


if __name__ == '__main__':
  main()



# pylint: disable=invalid-name
# How wide to allow formatted cmake files
line_width = 80

# How many spaces to tab for indent
tab_size = 2

# If true, separate control flow names from the parentheses with a space
separate_ctrl_name_with_space = False

# If true, separate function names from the parenthesis with a space
separate_fn_name_with_space = False

# Additional FLAGS and KWARGS for custom commands
additional_commands = {
    "foo": {
        "flags": ["BAR", "BAZ"],
        "kwargs": {
            "HEADERS": '*',
            "SOURCES": '*',
            "DEPENDS": '*',
        }
    }
}



from cmakelang.parse.additional_nodes import TupleParser
from cmakelang.parse.argument_nodes import StandardArgTree


def parse_set_target_properties(ctx, tokens, breakstack):
  """
  ::
    set_target_properties(target1 target2 ...
                        PROPERTIES prop1 value1
                        prop2 value2 ...)
  """

  return StandardArgTree.parse(
      ctx, tokens,
      npargs='+',
      kwargs={
          "PROPERTIES": TupleParser(2, '+', [])
      },
      flags=[],
      breakstack=breakstack)


def populate_db(parse_db):
  parse_db["set_target_properties"] = parse_set_target_properties



"""
Helper script called by cmake at install time to install debug binary
"""
import argparse
import logging
import os
import shutil
import subprocess
import sys


def main():
  parser = argparse.ArgumentParser(description=__doc__)
  parser.add_argument("--bin-path", help="path to the debug binary")
  parser.add_argument("--lib-dir", help="path to the install libdir")
  args = parser.parse_args()

  readelf_output = subprocess.check_output([
      "readelf", "-n", args.bin_path
  ]).decode("utf-8")

  lastline = readelf_output.strip().rsplit("\n", 1)[1]

  if not lastline.lstrip().startswith("Build ID:"):
    logging.error("readelf output doesn't end with BUILD ID")
    return 1
  buildid = lastline.split(":", 1)[1].strip()
  destdir = os.path.join(args.lib_dir, "debug/.build-id", buildid[:2])

  if "DESTDIR" in os.environ:
    destdir = os.path.join(os.environ["DESTDIR"], destdir.lstrip("/"))

  if not os.path.exists(destdir):
    os.makedirs(destdir)

  destpath = os.path.join(destdir, buildid[2:] + ".debug")
  if (os.path.exists(destpath) and
      os.path.getmtime(destpath) >= os.path.getmtime(args.bin_path)):
    print("-- Up-to-date: {}".format(destpath))
  else:
    print("-- Installing: {}".format(destpath))
    subprocess.check_call([
        "objcopy", "--only-keep-debug", args.bin_path, destpath])
  return 0


if __name__ == "__main__":
  logging.basicConfig(level=logging.WARNING)
  sys.exit(main())
